#!/usr/bin/env python3
# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

from __future__ import absolute_import
import os
import unittest
import sys

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
PARENT_DIR = os.path.join(SCRIPT_DIR, os.pardir)
sys.path.append(PARENT_DIR)
import low_bandwidth_audio_test


class TestExtractTestRuns(unittest.TestCase):
  def _TestLog(self, log, *expected):
    self.assertEqual(
        tuple(low_bandwidth_audio_test.ExtractTestRuns(log.splitlines(True))),
        expected)

  def testLinux(self):
    self._TestLog(
        LINUX_LOG,
        (None, 'GoodNetworkHighBitrate',
         '/webrtc/src/resources/voice_engine/audio_tiny16.wav',
         '/webrtc/src/out/LowBandwidth_GoodNetworkHighBitrate.wav', None),
        (None, 'Mobile2GNetwork',
         '/webrtc/src/resources/voice_engine/audio_tiny16.wav',
         '/webrtc/src/out/LowBandwidth_Mobile2GNetwork.wav', None),
        (None, 'PCGoodNetworkHighBitrate',
         '/webrtc/src/resources/voice_engine/audio_tiny16.wav',
         '/webrtc/src/out/PCLowBandwidth_PCGoodNetworkHighBitrate.wav',
         '/webrtc/src/out/PCLowBandwidth_perf_48.json'),
        (None, 'PCMobile2GNetwork',
         '/webrtc/src/resources/voice_engine/audio_tiny16.wav',
         '/webrtc/src/out/PCLowBandwidth_PCMobile2GNetwork.wav',
         '/webrtc/src/out/PCLowBandwidth_perf_48.json'))

  def testAndroid(self):
    self._TestLog(
        ANDROID_LOG,
        ('ddfa6149', 'Mobile2GNetwork',
         '/sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav',
         '/sdcard/chromium_tests_root/LowBandwidth_Mobile2GNetwork.wav', None),
        ('TA99205CNO', 'GoodNetworkHighBitrate',
         '/sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav',
         '/sdcard/chromium_tests_root/LowBandwidth_GoodNetworkHighBitrate.wav',
         None),
        ('ddfa6149', 'PCMobile2GNetwork',
         '/sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav',
         '/sdcard/chromium_tests_root/PCLowBandwidth_PCMobile2GNetwork.wav',
         '/sdcard/chromium_tests_root/PCLowBandwidth_perf_48.json'),
        ('TA99205CNO', 'PCGoodNetworkHighBitrate',
         '/sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav',
         ('/sdcard/chromium_tests_root/'
          'PCLowBandwidth_PCGoodNetworkHighBitrate.wav'),
         '/sdcard/chromium_tests_root/PCLowBandwidth_perf_48.json'))


LINUX_LOG = r'''\
[==========] Running 2 tests from 1 test case.
[----------] Global test environment set-up.
[----------] 2 tests from LowBandwidthAudioTest
[ RUN      ] LowBandwidthAudioTest.GoodNetworkHighBitrate
TEST GoodNetworkHighBitrate /webrtc/src/resources/voice_engine/audio_tiny16.wav /webrtc/src/out/LowBandwidth_GoodNetworkHighBitrate.wav
[       OK ] LowBandwidthAudioTest.GoodNetworkHighBitrate (5932 ms)
[ RUN      ] LowBandwidthAudioTest.Mobile2GNetwork
TEST Mobile2GNetwork /webrtc/src/resources/voice_engine/audio_tiny16.wav /webrtc/src/out/LowBandwidth_Mobile2GNetwork.wav
[       OK ] LowBandwidthAudioTest.Mobile2GNetwork (6333 ms)
[----------] 2 tests from LowBandwidthAudioTest (12265 ms total)
[----------] 2 tests from PCLowBandwidthAudioTest
[ RUN      ] PCLowBandwidthAudioTest.PCGoodNetworkHighBitrate
TEST PCGoodNetworkHighBitrate /webrtc/src/resources/voice_engine/audio_tiny16.wav /webrtc/src/out/PCLowBandwidth_PCGoodNetworkHighBitrate.wav /webrtc/src/out/PCLowBandwidth_perf_48.json
[       OK ] PCLowBandwidthAudioTest.PCGoodNetworkHighBitrate (5932 ms)
[ RUN      ] PCLowBandwidthAudioTest.PCMobile2GNetwork
TEST PCMobile2GNetwork /webrtc/src/resources/voice_engine/audio_tiny16.wav /webrtc/src/out/PCLowBandwidth_PCMobile2GNetwork.wav /webrtc/src/out/PCLowBandwidth_perf_48.json
[       OK ] PCLowBandwidthAudioTest.PCMobile2GNetwork (6333 ms)
[----------] 2 tests from PCLowBandwidthAudioTest (12265 ms total)

[----------] Global test environment tear-down
[==========] 2 tests from 1 test case ran. (12266 ms total)
[  PASSED  ] 2 tests.
'''

ANDROID_LOG = r'''\
I    0.000s Main  command: /webrtc/src/build/android/test_runner.py gtest --suite low_bandwidth_audio_test --output-directory /webrtc/src/out/debug-android --runtime-deps-path /webrtc/src/out/debug-android/gen.runtime/webrtc/audio/low_bandwidth_audio_test__test_runner_script.runtime_deps -v
I    0.007s Main  [host]> /webrtc/src/third_party/android_sdk/public/build-tools/24.0.2/aapt dump xmltree /webrtc/src/out/debug-android/low_bandwidth_audio_test_apk/low_bandwidth_audio_test-debug.apk AndroidManifest.xml
I    0.028s TimeoutThread-1-for-MainThread  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb devices
I    0.062s TimeoutThread-1-for-prepare_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO wait-for-device
I    0.063s TimeoutThread-1-for-prepare_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 wait-for-device
I    0.102s TimeoutThread-1-for-prepare_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( ( c=/data/local/tmp/cache_token;echo $EXTERNAL_STORAGE;cat $c 2>/dev/null||echo;echo "77611072-160c-11d7-9362-705b0f464195">$c &&getprop )>/data/local/tmp/temp_file-5ea34389e3f92 );echo %$?'
I    0.105s TimeoutThread-1-for-prepare_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( ( c=/data/local/tmp/cache_token;echo $EXTERNAL_STORAGE;cat $c 2>/dev/null||echo;echo "77618afc-160c-11d7-bda4-705b0f464195">$c &&getprop )>/data/local/tmp/temp_file-b995cef6e0e3d );echo %$?'
I    0.204s TimeoutThread-1-for-prepare_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 pull /data/local/tmp/temp_file-b995cef6e0e3d /tmp/tmpieAgDj/tmp_ReadFileWithPull
I    0.285s TimeoutThread-1-for-prepare_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( test -d /storage/emulated/legacy );echo %$?'
I    0.285s TimeoutThread-1-for-delete_temporary_file(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell 'rm -f /data/local/tmp/temp_file-b995cef6e0e3d'
I    0.302s TimeoutThread-1-for-prepare_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO pull /data/local/tmp/temp_file-5ea34389e3f92 /tmp/tmpvlyG3I/tmp_ReadFileWithPull
I    0.352s TimeoutThread-1-for-prepare_device(ddfa6149)  condition 'sd_card_ready' met (0.3s)
I    0.353s TimeoutThread-1-for-prepare_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( pm path android );echo %$?'
I    0.369s TimeoutThread-1-for-prepare_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( test -d /sdcard );echo %$?'
I    0.370s TimeoutThread-1-for-delete_temporary_file(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell 'rm -f /data/local/tmp/temp_file-5ea34389e3f92'
I    0.434s TimeoutThread-1-for-prepare_device(TA99205CNO)  condition 'sd_card_ready' met (0.4s)
I    0.434s TimeoutThread-1-for-prepare_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( pm path android );echo %$?'
I    1.067s TimeoutThread-1-for-prepare_device(ddfa6149)  condition 'pm_ready' met (1.0s)
I    1.067s TimeoutThread-1-for-prepare_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( getprop sys.boot_completed );echo %$?'
I    1.115s TimeoutThread-1-for-prepare_device(ddfa6149)  condition 'boot_completed' met (1.1s)
I    1.181s TimeoutThread-1-for-prepare_device(TA99205CNO)  condition 'pm_ready' met (1.1s)
I    1.181s TimeoutThread-1-for-prepare_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( getprop sys.boot_completed );echo %$?'
I    1.242s TimeoutThread-1-for-prepare_device(TA99205CNO)  condition 'boot_completed' met (1.2s)
I    1.268s TimeoutThread-1-for-individual_device_set_up(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( pm path org.chromium.native_test );echo %$?'
I    1.269s TimeoutThread-1-for-individual_device_set_up(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( pm path org.chromium.native_test );echo %$?'
I    2.008s calculate_device_checksums  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( a=/data/local/tmp/md5sum/md5sum_bin;! [[ $(ls -l $a) = *1225256* ]]&&exit 2;export LD_LIBRARY_PATH=/data/local/tmp/md5sum;$a /data/app/org.chromium.native_test-2/base.apk;: );echo %$?'
I    2.008s calculate_host_checksums  [host]> /webrtc/src/out/debug-android/md5sum_bin_host /webrtc/src/out/debug-android/low_bandwidth_audio_test_apk/low_bandwidth_audio_test-debug.apk
I    2.019s calculate_device_checksums  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( a=/data/local/tmp/md5sum/md5sum_bin;! [[ $(ls -l $a) = *1225256* ]]&&exit 2;export LD_LIBRARY_PATH=/data/local/tmp/md5sum;$a /data/app/org.chromium.native_test-1/base.apk;: );echo %$?'
I    2.020s calculate_host_checksums  [host]> /webrtc/src/out/debug-android/md5sum_bin_host /webrtc/src/out/debug-android/low_bandwidth_audio_test_apk/low_bandwidth_audio_test-debug.apk
I    2.172s TimeoutThread-1-for-individual_device_set_up(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( p=org.chromium.native_test;if [[ "$(ps)" = *$p* ]]; then am force-stop $p; fi );echo %$?'
I    2.183s TimeoutThread-1-for-individual_device_set_up(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( p=org.chromium.native_test;if [[ "$(ps)" = *$p* ]]; then am force-stop $p; fi );echo %$?'
I    2.290s calculate_device_checksums  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( a=/data/local/tmp/md5sum/md5sum_bin;! [[ $(ls -l $a) = *1225256* ]]&&exit 2;export LD_LIBRARY_PATH=/data/local/tmp/md5sum;$a /sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav;: );echo %$?'
I    2.291s calculate_host_checksums  [host]> /webrtc/src/out/debug-android/md5sum_bin_host /webrtc/src/resources/voice_engine/audio_tiny16.wav
I    2.373s calculate_device_checksums  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( a=/data/local/tmp/md5sum/md5sum_bin;! [[ $(ls -l $a) = *1225256* ]]&&exit 2;export LD_LIBRARY_PATH=/data/local/tmp/md5sum;$a /storage/emulated/legacy/chromium_tests_root/resources/voice_engine/audio_tiny16.wav;: );echo %$?'
I    2.374s calculate_host_checksums  [host]> /webrtc/src/out/debug-android/md5sum_bin_host /webrtc/src/resources/voice_engine/audio_tiny16.wav
I    2.390s calculate_device_checksums  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( a=/data/local/tmp/md5sum/md5sum_bin;! [[ $(ls -l $a) = *1225256* ]]&&exit 2;export LD_LIBRARY_PATH=/data/local/tmp/md5sum;$a /sdcard/chromium_tests_root/icudtl.dat;: );echo %$?'
I    2.390s calculate_host_checksums  [host]> /webrtc/src/out/debug-android/md5sum_bin_host /webrtc/src/out/debug-android/icudtl.dat
I    2.472s calculate_device_checksums  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( a=/data/local/tmp/md5sum/md5sum_bin;! [[ $(ls -l $a) = *1225256* ]]&&exit 2;export LD_LIBRARY_PATH=/data/local/tmp/md5sum;$a /storage/emulated/legacy/chromium_tests_root/icudtl.dat;: );echo %$?'
I    2.472s calculate_host_checksums  [host]> /webrtc/src/out/debug-android/md5sum_bin_host /webrtc/src/out/debug-android/icudtl.dat
I    2.675s TimeoutThread-1-for-list_tests(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( ( p=org.chromium.native_test;am instrument -w -e "$p".NativeTestInstrumentationTestRunner.ShardNanoTimeout 30000000000 -e "$p".NativeTestInstrumentationTestRunner.NativeTestActivity "$p".NativeUnitTestActivity -e "$p".NativeTestInstrumentationTestRunner.StdoutFile /sdcard/temp_file-6407c967884af.gtest_out -e "$p".NativeTest.CommandLineFlags --gtest_list_tests "$p"/"$p".NativeTestInstrumentationTestRunner )>/data/local/tmp/temp_file-d21ebcd0977d9 );echo %$?'
I    2.675s TimeoutThread-1-for-list_tests(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( ( p=org.chromium.native_test;am instrument -w -e "$p".NativeTestInstrumentationTestRunner.ShardNanoTimeout 30000000000 -e "$p".NativeTestInstrumentationTestRunner.NativeTestActivity "$p".NativeUnitTestActivity -e "$p".NativeTestInstrumentationTestRunner.StdoutFile /storage/emulated/legacy/temp_file-fa09560c3259.gtest_out -e "$p".NativeTest.CommandLineFlags --gtest_list_tests "$p"/"$p".NativeTestInstrumentationTestRunner )>/data/local/tmp/temp_file-95ad995999939 );echo %$?'
I    3.739s TimeoutThread-1-for-list_tests(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 pull /data/local/tmp/temp_file-95ad995999939 /tmp/tmpSnnF6Y/tmp_ReadFileWithPull
I    3.807s TimeoutThread-1-for-delete_temporary_file(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell 'rm -f /data/local/tmp/temp_file-95ad995999939'
I    3.812s TimeoutThread-1-for-list_tests(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( TZ=utc ls -a -l /storage/emulated/legacy/ );echo %$?'
I    3.866s TimeoutThread-1-for-list_tests(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( cat /storage/emulated/legacy/temp_file-fa09560c3259.gtest_out );echo %$?'
I    3.912s TimeoutThread-1-for-delete_temporary_file(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell 'rm -f /storage/emulated/legacy/temp_file-fa09560c3259.gtest_out'
I    4.256s TimeoutThread-1-for-list_tests(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO pull /data/local/tmp/temp_file-d21ebcd0977d9 /tmp/tmpokPF5b/tmp_ReadFileWithPull
I    4.324s TimeoutThread-1-for-delete_temporary_file(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell 'rm -f /data/local/tmp/temp_file-d21ebcd0977d9'
I    4.342s TimeoutThread-1-for-list_tests(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( TZ=utc ls -a -l /sdcard/ );echo %$?'
I    4.432s TimeoutThread-1-for-list_tests(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( cat /sdcard/temp_file-6407c967884af.gtest_out );echo %$?'
I    4.476s TimeoutThread-1-for-delete_temporary_file(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell 'rm -f /sdcard/temp_file-6407c967884af.gtest_out'
I    4.483s Main  Using external sharding settings. This is shard 0/1
I    4.483s Main  STARTING TRY #1/3
I    4.484s Main  Will run 2 tests on 2 devices: TA99205CNO, ddfa6149
I    4.486s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( pm dump org.chromium.native_test | grep dataDir=; echo "PIPESTATUS: ${PIPESTATUS[@]}" );echo %$?'
I    4.486s TimeoutThread-1-for-run_tests_on_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( pm dump org.chromium.native_test | grep dataDir=; echo "PIPESTATUS: ${PIPESTATUS[@]}" );echo %$?'
I    5.551s run_tests_on_device(TA99205CNO)  flags:
I    5.552s run_tests_on_device(ddfa6149)  flags:
I    5.554s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( ( p=org.chromium.native_test;am instrument -w -e "$p".NativeTestInstrumentationTestRunner.ShardNanoTimeout 120000000000 -e "$p".NativeTestInstrumentationTestRunner.NativeTestActivity "$p".NativeUnitTestActivity -e "$p".NativeTestInstrumentationTestRunner.Test LowBandwidthAudioTest.GoodNetworkHighBitrate -e "$p".NativeTestInstrumentationTestRunner.StdoutFile /sdcard/temp_file-ffe7b76691cb7.gtest_out "$p"/"$p".NativeTestInstrumentationTestRunner )>/data/local/tmp/temp_file-c9d83b3078ab1 );echo %$?'
I    5.556s TimeoutThread-1-for-run_tests_on_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( ( p=org.chromium.native_test;am instrument -w -e "$p".NativeTestInstrumentationTestRunner.ShardNanoTimeout 120000000000 -e "$p".NativeTestInstrumentationTestRunner.NativeTestActivity "$p".NativeUnitTestActivity -e "$p".NativeTestInstrumentationTestRunner.Test LowBandwidthAudioTest.Mobile2GNetwork -e "$p".NativeTestInstrumentationTestRunner.StdoutFile /storage/emulated/legacy/temp_file-f0ceb1a05ea8.gtest_out "$p"/"$p".NativeTestInstrumentationTestRunner )>/data/local/tmp/temp_file-245ef307a5b32 );echo %$?'
I   12.956s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO pull /data/local/tmp/temp_file-c9d83b3078ab1 /tmp/tmpRQhTcM/tmp_ReadFileWithPull
I   13.024s TimeoutThread-1-for-delete_temporary_file(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell 'rm -f /data/local/tmp/temp_file-c9d83b3078ab1'
I   13.032s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( TZ=utc ls -a -l /sdcard/ );echo %$?'
I   13.114s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( cat /sdcard/temp_file-ffe7b76691cb7.gtest_out );echo %$?'
I   13.154s TimeoutThread-1-for-run_tests_on_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 pull /data/local/tmp/temp_file-245ef307a5b32 /tmp/tmpfQ4J96/tmp_ReadFileWithPull
I   13.167s TimeoutThread-1-for-delete_temporary_file(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell 'rm -f /sdcard/temp_file-ffe7b76691cb7.gtest_out'
I   13.169s TimeoutThread-1-for-delete_temporary_file(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell 'rm -f /data/user/0/org.chromium.native_test/temp_file-f07c4808dbf8f.xml'
I   13.170s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( pm clear org.chromium.native_test );echo %$?'
I   13.234s TimeoutThread-1-for-delete_temporary_file(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell 'rm -f /data/local/tmp/temp_file-245ef307a5b32'
I   13.239s TimeoutThread-1-for-run_tests_on_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( TZ=utc ls -a -l /storage/emulated/legacy/ );echo %$?'
I   13.291s TimeoutThread-1-for-run_tests_on_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( cat /storage/emulated/legacy/temp_file-f0ceb1a05ea8.gtest_out );echo %$?'
I   13.341s TimeoutThread-1-for-delete_temporary_file(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell 'rm -f /storage/emulated/legacy/temp_file-f0ceb1a05ea8.gtest_out'
I   13.343s TimeoutThread-1-for-delete_temporary_file(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell 'rm -f /data/data/org.chromium.native_test/temp_file-5649bb01682da.xml'
I   13.346s TimeoutThread-1-for-run_tests_on_device(ddfa6149)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s ddfa6149 shell '( pm clear org.chromium.native_test );echo %$?'
I   13.971s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  Setting permissions for org.chromium.native_test.
I   13.971s TimeoutThread-1-for-run_tests_on_device(TA99205CNO)  [host]> /webrtc/src/third_party/android_sdk/public/platform-tools/adb -s TA99205CNO shell '( pm grant org.chromium.native_test android.permission.CAMERA&&pm grant org.chromium.native_test android.permission.RECORD_AUDIO&&pm grant org.chromium.native_test android.permission.WRITE_EXTERNAL_STORAGE&&pm grant org.chromium.native_test android.permission.READ_EXTERNAL_STORAGE );echo %$?'
I   14.078s run_tests_on_device(ddfa6149)  >>ScopedMainEntryLogger
I   14.078s run_tests_on_device(ddfa6149)  Note: Google Test filter = LowBandwidthAudioTest.Mobile2GNetwork
I   14.078s run_tests_on_device(ddfa6149)  [==========] Running 1 test from 1 test case.
I   14.078s run_tests_on_device(ddfa6149)  [----------] Global test environment set-up.
I   14.078s run_tests_on_device(ddfa6149)  [----------] 1 test from LowBandwidthAudioTest
I   14.078s run_tests_on_device(ddfa6149)  [ RUN      ] LowBandwidthAudioTest.Mobile2GNetwork
I   14.078s run_tests_on_device(ddfa6149)  TEST Mobile2GNetwork /sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav /sdcard/chromium_tests_root/LowBandwidth_Mobile2GNetwork.wav
I   14.078s run_tests_on_device(ddfa6149)  [       OK ] LowBandwidthAudioTest.Mobile2GNetwork (6438 ms)
I   14.078s run_tests_on_device(ddfa6149)  [----------] 1 test from LowBandwidthAudioTest (6438 ms total)
I   14.078s run_tests_on_device(ddfa6149)
I   14.078s run_tests_on_device(ddfa6149)  [----------] Global test environment tear-down
I   14.079s run_tests_on_device(ddfa6149)  [==========] 1 test from 1 test case ran. (6438 ms total)
I   14.079s run_tests_on_device(ddfa6149)  [  PASSED  ] 1 test.
I   14.079s run_tests_on_device(ddfa6149)  <<ScopedMainEntryLogger
I   16.576s run_tests_on_device(TA99205CNO)  >>ScopedMainEntryLogger
I   16.576s run_tests_on_device(TA99205CNO)  Note: Google Test filter = LowBandwidthAudioTest.GoodNetworkHighBitrate
I   16.576s run_tests_on_device(TA99205CNO)  [==========] Running 1 test from 1 test case.
I   16.576s run_tests_on_device(TA99205CNO)  [----------] Global test environment set-up.
I   16.576s run_tests_on_device(TA99205CNO)  [----------] 1 test from LowBandwidthAudioTest
I   16.576s run_tests_on_device(TA99205CNO)  [ RUN      ] LowBandwidthAudioTest.GoodNetworkHighBitrate
I   16.576s run_tests_on_device(TA99205CNO)  TEST GoodNetworkHighBitrate /sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav /sdcard/chromium_tests_root/LowBandwidth_GoodNetworkHighBitrate.wav
I   16.576s run_tests_on_device(TA99205CNO)  [       OK ] LowBandwidthAudioTest.GoodNetworkHighBitrate (5968 ms)
I   16.576s run_tests_on_device(TA99205CNO)  [----------] 1 test from LowBandwidthAudioTest (5968 ms total)
I   16.576s run_tests_on_device(TA99205CNO)
I   16.576s run_tests_on_device(TA99205CNO)  [----------] Global test environment tear-down
I   16.576s run_tests_on_device(TA99205CNO)  [==========] 1 test from 1 test case ran. (5968 ms total)
I   16.577s run_tests_on_device(TA99205CNO)  [  PASSED  ] 1 test.
I   16.577s run_tests_on_device(TA99205CNO)  <<ScopedMainEntryLogger
I   14.078s run_tests_on_device(ddfa6149)  >>ScopedMainEntryLogger
I   14.078s run_tests_on_device(ddfa6149)  Note: Google Test filter = PCLowBandwidthAudioTest.PCMobile2GNetwork
I   14.078s run_tests_on_device(ddfa6149)  [==========] Running 1 test from 1 test case.
I   14.078s run_tests_on_device(ddfa6149)  [----------] Global test environment set-up.
I   14.078s run_tests_on_device(ddfa6149)  [----------] 1 test from PCLowBandwidthAudioTest
I   14.078s run_tests_on_device(ddfa6149)  [ RUN      ] PCLowBandwidthAudioTest.PCMobile2GNetwork
I   14.078s run_tests_on_device(ddfa6149)  TEST PCMobile2GNetwork /sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav /sdcard/chromium_tests_root/PCLowBandwidth_PCMobile2GNetwork.wav /sdcard/chromium_tests_root/PCLowBandwidth_perf_48.json
I   14.078s run_tests_on_device(ddfa6149)  [       OK ] PCLowBandwidthAudioTest.PCMobile2GNetwork (6438 ms)
I   14.078s run_tests_on_device(ddfa6149)  [----------] 1 test from PCLowBandwidthAudioTest (6438 ms total)
I   14.078s run_tests_on_device(ddfa6149)
I   14.078s run_tests_on_device(ddfa6149)  [----------] Global test environment tear-down
I   14.079s run_tests_on_device(ddfa6149)  [==========] 1 test from 1 test case ran. (6438 ms total)
I   14.079s run_tests_on_device(ddfa6149)  [  PASSED  ] 1 test.
I   14.079s run_tests_on_device(ddfa6149)  <<ScopedMainEntryLogger
I   16.576s run_tests_on_device(TA99205CNO)  >>ScopedMainEntryLogger
I   16.576s run_tests_on_device(TA99205CNO)  Note: Google Test filter = PCLowBandwidthAudioTest.PCGoodNetworkHighBitrate
I   16.576s run_tests_on_device(TA99205CNO)  [==========] Running 1 test from 1 test case.
I   16.576s run_tests_on_device(TA99205CNO)  [----------] Global test environment set-up.
I   16.576s run_tests_on_device(TA99205CNO)  [----------] 1 test from PCLowBandwidthAudioTest
I   16.576s run_tests_on_device(TA99205CNO)  [ RUN      ] PCLowBandwidthAudioTest.PCGoodNetworkHighBitrate
I   16.576s run_tests_on_device(TA99205CNO)  TEST PCGoodNetworkHighBitrate /sdcard/chromium_tests_root/resources/voice_engine/audio_tiny16.wav /sdcard/chromium_tests_root/PCLowBandwidth_PCGoodNetworkHighBitrate.wav /sdcard/chromium_tests_root/PCLowBandwidth_perf_48.json
I   16.576s run_tests_on_device(TA99205CNO)  [       OK ] PCLowBandwidthAudioTest.PCGoodNetworkHighBitrate (5968 ms)
I   16.576s run_tests_on_device(TA99205CNO)  [----------] 1 test from PCLowBandwidthAudioTest (5968 ms total)
I   16.576s run_tests_on_device(TA99205CNO)
I   16.576s run_tests_on_device(TA99205CNO)  [----------] Global test environment tear-down
I   16.576s run_tests_on_device(TA99205CNO)  [==========] 1 test from 1 test case ran. (5968 ms total)
I   16.577s run_tests_on_device(TA99205CNO)  [  PASSED  ] 1 test.
I   16.577s run_tests_on_device(TA99205CNO)  <<ScopedMainEntryLogger
I   16.577s run_tests_on_device(TA99205CNO)  Finished running tests on this device.
I   16.577s run_tests_on_device(ddfa6149)  Finished running tests on this device.
I   16.604s Main  FINISHED TRY #1/3
I   16.604s Main  All tests completed.
C   16.604s Main  ********************************************************************************
C   16.604s Main  Summary
C   16.604s Main  ********************************************************************************
C   16.605s Main  [==========] 2 tests ran.
C   16.605s Main  [  PASSED  ] 2 tests.
C   16.605s Main  ********************************************************************************
I   16.608s tear_down_device(ddfa6149)  Wrote device cache: /webrtc/src/out/debug-android/device_cache_ddea6549.json
I   16.608s tear_down_device(TA99205CNO)  Wrote device cache: /webrtc/src/out/debug-android/device_cache_TA99305CMO.json
'''

if __name__ == "__main__":
  unittest.main()



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Test data generators producing signals pairs intended to be used to
test the APM module. Each pair consists of a noisy input and a reference signal.
The former is used as APM input and it is generated by adding noise to a
clean audio track. The reference is the expected APM output.

Throughout this file, the following naming convention is used:
  - input signal: the clean signal (e.g., speech),
  - noise signal: the noise to be summed up to the input signal (e.g., white
    noise, Gaussian noise),
  - noisy signal: input + noise.
The noise signal may or may not be a function of the clean signal. For
instance, white noise is independently generated, whereas reverberation is
obtained by convolving the input signal with an impulse response.
"""

import logging
import os
import shutil
import sys

try:
    import scipy.io
except ImportError:
    logging.critical('Cannot import the third-party Python package scipy')
    sys.exit(1)

from . import data_access
from . import exceptions
from . import signal_processing


class TestDataGenerator(object):
    """Abstract class responsible for the generation of noisy signals.

  Given a clean signal, it generates two streams named noisy signal and
  reference. The former is the clean signal deteriorated by the noise source,
  the latter goes through the same deterioration process, but more "gently".
  Noisy signal and reference are produced so that the reference is the signal
  expected at the output of the APM module when the latter is fed with the noisy
  signal.

  An test data generator generates one or more pairs.
  """

    NAME = None
    REGISTERED_CLASSES = {}

    def __init__(self, output_directory_prefix):
        self._output_directory_prefix = output_directory_prefix
        # Init dictionaries with one entry for each test data generator
        # configuration (e.g., different SNRs).
        # Noisy audio track files (stored separately in a cache folder).
        self._noisy_signal_filepaths = None
        # Path to be used for the APM simulation output files.
        self._apm_output_paths = None
        # Reference audio track files (stored separately in a cache folder).
        self._reference_signal_filepaths = None
        self.Clear()

    @classmethod
    def RegisterClass(cls, class_to_register):
        """Registers a TestDataGenerator implementation.

    Decorator to automatically register the classes that extend
    TestDataGenerator.
    Example usage:

    @TestDataGenerator.RegisterClass
    class IdentityGenerator(TestDataGenerator):
      pass
    """
        cls.REGISTERED_CLASSES[class_to_register.NAME] = class_to_register
        return class_to_register

    @property
    def config_names(self):
        return self._noisy_signal_filepaths.keys()

    @property
    def noisy_signal_filepaths(self):
        return self._noisy_signal_filepaths

    @property
    def apm_output_paths(self):
        return self._apm_output_paths

    @property
    def reference_signal_filepaths(self):
        return self._reference_signal_filepaths

    def Generate(self, input_signal_filepath, test_data_cache_path,
                 base_output_path):
        """Generates a set of noisy input and reference audiotrack file pairs.

    This method initializes an empty set of pairs and calls the _Generate()
    method implemented in a concrete class.

    Args:
      input_signal_filepath: path to the clean input audio track file.
      test_data_cache_path: path to the cache of the generated audio track
                            files.
      base_output_path: base path where output is written.
    """
        self.Clear()
        self._Generate(input_signal_filepath, test_data_cache_path,
                       base_output_path)

    def Clear(self):
        """Clears the generated output path dictionaries.
    """
        self._noisy_signal_filepaths = {}
        self._apm_output_paths = {}
        self._reference_signal_filepaths = {}

    def _Generate(self, input_signal_filepath, test_data_cache_path,
                  base_output_path):
        """Abstract method to be implemented in each concrete class.
    """
        raise NotImplementedError()

    def _AddNoiseSnrPairs(self, base_output_path, noisy_mix_filepaths,
                          snr_value_pairs):
        """Adds noisy-reference signal pairs.

    Args:
      base_output_path: noisy tracks base output path.
      noisy_mix_filepaths: nested dictionary of noisy signal paths organized
                           by noisy track name and SNR level.
      snr_value_pairs: list of SNR pairs.
    """
        for noise_track_name in noisy_mix_filepaths:
            for snr_noisy, snr_refence in snr_value_pairs:
                config_name = '{0}_{1:d}_{2:d}_SNR'.format(
                    noise_track_name, snr_noisy, snr_refence)
                output_path = self._MakeDir(base_output_path, config_name)
                self._AddNoiseReferenceFilesPair(
                    config_name=config_name,
                    noisy_signal_filepath=noisy_mix_filepaths[noise_track_name]
                    [snr_noisy],
                    reference_signal_filepath=noisy_mix_filepaths[
                        noise_track_name][snr_refence],
                    output_path=output_path)

    def _AddNoiseReferenceFilesPair(self, config_name, noisy_signal_filepath,
                                    reference_signal_filepath, output_path):
        """Adds one noisy-reference signal pair.

    Args:
      config_name: name of the APM configuration.
      noisy_signal_filepath: path to noisy audio track file.
      reference_signal_filepath: path to reference audio track file.
      output_path: APM output path.
    """
        assert config_name not in self._noisy_signal_filepaths
        self._noisy_signal_filepaths[config_name] = os.path.abspath(
            noisy_signal_filepath)
        self._apm_output_paths[config_name] = os.path.abspath(output_path)
        self._reference_signal_filepaths[config_name] = os.path.abspath(
            reference_signal_filepath)

    def _MakeDir(self, base_output_path, test_data_generator_config_name):
        output_path = os.path.join(
            base_output_path,
            self._output_directory_prefix + test_data_generator_config_name)
        data_access.MakeDirectory(output_path)
        return output_path


@TestDataGenerator.RegisterClass
class IdentityTestDataGenerator(TestDataGenerator):
    """Generator that adds no noise.

  Both the noisy and the reference signals are the input signal.
  """

    NAME = 'identity'

    def __init__(self, output_directory_prefix, copy_with_identity):
        TestDataGenerator.__init__(self, output_directory_prefix)
        self._copy_with_identity = copy_with_identity

    @property
    def copy_with_identity(self):
        return self._copy_with_identity

    def _Generate(self, input_signal_filepath, test_data_cache_path,
                  base_output_path):
        config_name = 'default'
        output_path = self._MakeDir(base_output_path, config_name)

        if self._copy_with_identity:
            input_signal_filepath_new = os.path.join(
                test_data_cache_path,
                os.path.split(input_signal_filepath)[1])
            logging.info('copying ' + input_signal_filepath + ' to ' +
                         (input_signal_filepath_new))
            shutil.copy(input_signal_filepath, input_signal_filepath_new)
            input_signal_filepath = input_signal_filepath_new

        self._AddNoiseReferenceFilesPair(
            config_name=config_name,
            noisy_signal_filepath=input_signal_filepath,
            reference_signal_filepath=input_signal_filepath,
            output_path=output_path)


@TestDataGenerator.RegisterClass
class WhiteNoiseTestDataGenerator(TestDataGenerator):
    """Generator that adds white noise.
  """

    NAME = 'white_noise'

    # Each pair indicates the clean vs. noisy and reference vs. noisy SNRs.
    # The reference (second value of each pair) always has a lower amount of noise
    # - i.e., the SNR is 10 dB higher.
    _SNR_VALUE_PAIRS = [
        [20, 30],  # Smallest noise.
        [10, 20],
        [5, 15],
        [0, 10],  # Largest noise.
    ]

    _NOISY_SIGNAL_FILENAME_TEMPLATE = 'noise_{0:d}_SNR.wav'

    def __init__(self, output_directory_prefix):
        TestDataGenerator.__init__(self, output_directory_prefix)

    def _Generate(self, input_signal_filepath, test_data_cache_path,
                  base_output_path):
        # Load the input signal.
        input_signal = signal_processing.SignalProcessingUtils.LoadWav(
            input_signal_filepath)

        # Create the noise track.
        noise_signal = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            input_signal)

        # Create the noisy mixes (once for each unique SNR value).
        noisy_mix_filepaths = {}
        snr_values = set(
            [snr for pair in self._SNR_VALUE_PAIRS for snr in pair])
        for snr in snr_values:
            noisy_signal_filepath = os.path.join(
                test_data_cache_path,
                self._NOISY_SIGNAL_FILENAME_TEMPLATE.format(snr))

            # Create and save if not done.
            if not os.path.exists(noisy_signal_filepath):
                # Create noisy signal.
                noisy_signal = signal_processing.SignalProcessingUtils.MixSignals(
                    input_signal, noise_signal, snr)

                # Save.
                signal_processing.SignalProcessingUtils.SaveWav(
                    noisy_signal_filepath, noisy_signal)

            # Add file to the collection of mixes.
            noisy_mix_filepaths[snr] = noisy_signal_filepath

        # Add all the noisy-reference signal pairs.
        for snr_noisy, snr_refence in self._SNR_VALUE_PAIRS:
            config_name = '{0:d}_{1:d}_SNR'.format(snr_noisy, snr_refence)
            output_path = self._MakeDir(base_output_path, config_name)
            self._AddNoiseReferenceFilesPair(
                config_name=config_name,
                noisy_signal_filepath=noisy_mix_filepaths[snr_noisy],
                reference_signal_filepath=noisy_mix_filepaths[snr_refence],
                output_path=output_path)


# TODO(alessiob): remove comment when class implemented.
# @TestDataGenerator.RegisterClass
class NarrowBandNoiseTestDataGenerator(TestDataGenerator):
    """Generator that adds narrow-band noise.
  """

    NAME = 'narrow_band_noise'

    def __init__(self, output_directory_prefix):
        TestDataGenerator.__init__(self, output_directory_prefix)

    def _Generate(self, input_signal_filepath, test_data_cache_path,
                  base_output_path):
        # TODO(alessiob): implement.
        pass


@TestDataGenerator.RegisterClass
class AdditiveNoiseTestDataGenerator(TestDataGenerator):
    """Generator that adds noise loops.

  This generator uses all the wav files in a given path (default: noise_tracks/)
  and mixes them to the clean speech with different target SNRs (hard-coded).
  """

    NAME = 'additive_noise'
    _NOISY_SIGNAL_FILENAME_TEMPLATE = '{0}_{1:d}_SNR.wav'

    DEFAULT_NOISE_TRACKS_PATH = os.path.join(os.path.dirname(__file__),
                                             os.pardir, 'noise_tracks')

    # TODO(alessiob): Make the list of SNR pairs customizable.
    # Each pair indicates the clean vs. noisy and reference vs. noisy SNRs.
    # The reference (second value of each pair) always has a lower amount of noise
    # - i.e., the SNR is 10 dB higher.
    _SNR_VALUE_PAIRS = [
        [20, 30],  # Smallest noise.
        [10, 20],
        [5, 15],
        [0, 10],  # Largest noise.
    ]

    def __init__(self, output_directory_prefix, noise_tracks_path):
        TestDataGenerator.__init__(self, output_directory_prefix)
        self._noise_tracks_path = noise_tracks_path
        self._noise_tracks_file_names = [
            n for n in os.listdir(self._noise_tracks_path)
            if n.lower().endswith('.wav')
        ]
        if len(self._noise_tracks_file_names) == 0:
            raise exceptions.InitializationException(
                'No wav files found in the noise tracks path %s' %
                (self._noise_tracks_path))

    def _Generate(self, input_signal_filepath, test_data_cache_path,
                  base_output_path):
        """Generates test data pairs using environmental noise.

    For each noise track and pair of SNR values, the following two audio tracks
    are created: the noisy signal and the reference signal. The former is
    obtained by mixing the (clean) input signal to the corresponding noise
    track enforcing the target SNR.
    """
        # Init.
        snr_values = set(
            [snr for pair in self._SNR_VALUE_PAIRS for snr in pair])

        # Load the input signal.
        input_signal = signal_processing.SignalProcessingUtils.LoadWav(
            input_signal_filepath)

        noisy_mix_filepaths = {}
        for noise_track_filename in self._noise_tracks_file_names:
            # Load the noise track.
            noise_track_name, _ = os.path.splitext(noise_track_filename)
            noise_track_filepath = os.path.join(self._noise_tracks_path,
                                                noise_track_filename)
            if not os.path.exists(noise_track_filepath):
                logging.error('cannot find the <%s> noise track',
                              noise_track_filename)
                raise exceptions.FileNotFoundError()

            noise_signal = signal_processing.SignalProcessingUtils.LoadWav(
                noise_track_filepath)

            # Create the noisy mixes (once for each unique SNR value).
            noisy_mix_filepaths[noise_track_name] = {}
            for snr in snr_values:
                noisy_signal_filepath = os.path.join(
                    test_data_cache_path,
                    self._NOISY_SIGNAL_FILENAME_TEMPLATE.format(
                        noise_track_name, snr))

                # Create and save if not done.
                if not os.path.exists(noisy_signal_filepath):
                    # Create noisy signal.
                    noisy_signal = signal_processing.SignalProcessingUtils.MixSignals(
                        input_signal,
                        noise_signal,
                        snr,
                        pad_noise=signal_processing.SignalProcessingUtils.
                        MixPadding.LOOP)

                    # Save.
                    signal_processing.SignalProcessingUtils.SaveWav(
                        noisy_signal_filepath, noisy_signal)

                # Add file to the collection of mixes.
                noisy_mix_filepaths[noise_track_name][
                    snr] = noisy_signal_filepath

        # Add all the noise-SNR pairs.
        self._AddNoiseSnrPairs(base_output_path, noisy_mix_filepaths,
                               self._SNR_VALUE_PAIRS)


@TestDataGenerator.RegisterClass
class ReverberationTestDataGenerator(TestDataGenerator):
    """Generator that adds reverberation noise.

  TODO(alessiob): Make this class more generic since the impulse response can be
  anything (not just reverberation); call it e.g.,
  ConvolutionalNoiseTestDataGenerator.
  """

    NAME = 'reverberation'

    _IMPULSE_RESPONSES = {
        'lecture': 'air_binaural_lecture_0_0_1.mat',  # Long echo.
        'booth': 'air_binaural_booth_0_0_1.mat',  # Short echo.
    }
    _MAX_IMPULSE_RESPONSE_LENGTH = None

    # Each pair indicates the clean vs. noisy and reference vs. noisy SNRs.
    # The reference (second value of each pair) always has a lower amount of noise
    # - i.e., the SNR is 5 dB higher.
    _SNR_VALUE_PAIRS = [
        [3, 8],  # Smallest noise.
        [-3, 2],  # Largest noise.
    ]

    _NOISE_TRACK_FILENAME_TEMPLATE = '{0}.wav'
    _NOISY_SIGNAL_FILENAME_TEMPLATE = '{0}_{1:d}_SNR.wav'

    def __init__(self, output_directory_prefix, aechen_ir_database_path):
        TestDataGenerator.__init__(self, output_directory_prefix)
        self._aechen_ir_database_path = aechen_ir_database_path

    def _Generate(self, input_signal_filepath, test_data_cache_path,
                  base_output_path):
        """Generates test data pairs using reverberation noise.

    For each impulse response, one noise track is created. For each impulse
    response and pair of SNR values, the following 2 audio tracks are
    created: the noisy signal and the reference signal. The former is
    obtained by mixing the (clean) input signal to the corresponding noise
    track enforcing the target SNR.
    """
        # Init.
        snr_values = set(
            [snr for pair in self._SNR_VALUE_PAIRS for snr in pair])

        # Load the input signal.
        input_signal = signal_processing.SignalProcessingUtils.LoadWav(
            input_signal_filepath)

        noisy_mix_filepaths = {}
        for impulse_response_name in self._IMPULSE_RESPONSES:
            noise_track_filename = self._NOISE_TRACK_FILENAME_TEMPLATE.format(
                impulse_response_name)
            noise_track_filepath = os.path.join(test_data_cache_path,
                                                noise_track_filename)
            noise_signal = None
            try:
                # Load noise track.
                noise_signal = signal_processing.SignalProcessingUtils.LoadWav(
                    noise_track_filepath)
            except exceptions.FileNotFoundError:
                # Generate noise track by applying the impulse response.
                impulse_response_filepath = os.path.join(
                    self._aechen_ir_database_path,
                    self._IMPULSE_RESPONSES[impulse_response_name])
                noise_signal = self._GenerateNoiseTrack(
                    noise_track_filepath, input_signal,
                    impulse_response_filepath)
            assert noise_signal is not None

            # Create the noisy mixes (once for each unique SNR value).
            noisy_mix_filepaths[impulse_response_name] = {}
            for snr in snr_values:
                noisy_signal_filepath = os.path.join(
                    test_data_cache_path,
                    self._NOISY_SIGNAL_FILENAME_TEMPLATE.format(
                        impulse_response_name, snr))

                # Create and save if not done.
                if not os.path.exists(noisy_signal_filepath):
                    # Create noisy signal.
                    noisy_signal = signal_processing.SignalProcessingUtils.MixSignals(
                        input_signal, noise_signal, snr)

                    # Save.
                    signal_processing.SignalProcessingUtils.SaveWav(
                        noisy_signal_filepath, noisy_signal)

                # Add file to the collection of mixes.
                noisy_mix_filepaths[impulse_response_name][
                    snr] = noisy_signal_filepath

        # Add all the noise-SNR pairs.
        self._AddNoiseSnrPairs(base_output_path, noisy_mix_filepaths,
                               self._SNR_VALUE_PAIRS)

    def _GenerateNoiseTrack(self, noise_track_filepath, input_signal,
                            impulse_response_filepath):
        """Generates noise track.

    Generate a signal by convolving input_signal with the impulse response in
    impulse_response_filepath; then save to noise_track_filepath.

    Args:
      noise_track_filepath: output file path for the noise track.
      input_signal: (clean) input signal samples.
      impulse_response_filepath: impulse response file path.

    Returns:
      AudioSegment instance.
    """
        # Load impulse response.
        data = scipy.io.loadmat(impulse_response_filepath)
        impulse_response = data['h_air'].flatten()
        if self._MAX_IMPULSE_RESPONSE_LENGTH is not None:
            logging.info('truncating impulse response from %d to %d samples',
                         len(impulse_response),
                         self._MAX_IMPULSE_RESPONSE_LENGTH)
            impulse_response = impulse_response[:self.
                                                _MAX_IMPULSE_RESPONSE_LENGTH]

        # Apply impulse response.
        processed_signal = (
            signal_processing.SignalProcessingUtils.ApplyImpulseResponse(
                input_signal, impulse_response))

        # Save.
        signal_processing.SignalProcessingUtils.SaveWav(
            noise_track_filepath, processed_signal)

        return processed_signal



#!/usr/bin/python2
#  Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
#  Use of this source code is governed by a BSD-style license
#  that can be found in the LICENSE file in the root of the source
#  tree. An additional intellectual property rights grant can be found
#  in the file PATENTS.  All contributing project authors may
#  be found in the AUTHORS file in the root of the source tree.

#  To run this script please copy "out/<build_name>//pyproto/webrtc/modules/
#  audio_coding/audio_network_adaptor/debug_dump_pb2.py" to this folder.
#  The you can run this script with:
#  "python parse_ana_dump.py -m uplink_bandwidth_bps -f dump_file.dat"
#  You can add as may metrics or decisions to the plot as you like.
#  form more information call:
#  "python parse_ana_dump.py --help"

import struct
from optparse import OptionParser

import matplotlib.pyplot as plt

import debug_dump_pb2


def GetNextMessageSize(file_to_parse):
    data = file_to_parse.read(4)
    if data == '':
        return 0
    return struct.unpack('<I', data)[0]


def GetNextMessageFromFile(file_to_parse):
    message_size = GetNextMessageSize(file_to_parse)
    if message_size == 0:
        return None
    try:
        event = debug_dump_pb2.Event()
        event.ParseFromString(file_to_parse.read(message_size))
    except IOError:
        print 'Invalid message in file'
        return None
    return event


def InitMetrics():
    metrics = {}
    event = debug_dump_pb2.Event()
    for metric in event.network_metrics.DESCRIPTOR.fields:
        metrics[metric.name] = {'time': [], 'value': []}
    return metrics


def InitDecisions():
    decisions = {}
    event = debug_dump_pb2.Event()
    for decision in event.encoder_runtime_config.DESCRIPTOR.fields:
        decisions[decision.name] = {'time': [], 'value': []}
    return decisions


def ParseAnaDump(dump_file_to_parse):
    with open(dump_file_to_parse, 'rb') as file_to_parse:
        metrics = InitMetrics()
        decisions = InitDecisions()
        first_time_stamp = None
        while True:
            event = GetNextMessageFromFile(file_to_parse)
            if event is None:
                break
            if first_time_stamp is None:
                first_time_stamp = event.timestamp
            if event.type == debug_dump_pb2.Event.ENCODER_RUNTIME_CONFIG:
                for decision in event.encoder_runtime_config.DESCRIPTOR.fields:
                    if event.encoder_runtime_config.HasField(decision.name):
                        decisions[decision.name]['time'].append(
                            event.timestamp - first_time_stamp)
                        decisions[decision.name]['value'].append(
                            getattr(event.encoder_runtime_config,
                                    decision.name))
            if event.type == debug_dump_pb2.Event.NETWORK_METRICS:
                for metric in event.network_metrics.DESCRIPTOR.fields:
                    if event.network_metrics.HasField(metric.name):
                        metrics[metric.name]['time'].append(event.timestamp -
                                                            first_time_stamp)
                        metrics[metric.name]['value'].append(
                            getattr(event.network_metrics, metric.name))
    return (metrics, decisions)


def main():
    parser = OptionParser()
    parser.add_option("-f",
                      "--dump_file",
                      dest="dump_file_to_parse",
                      help="dump file to parse")
    parser.add_option('-m',
                      '--metric_plot',
                      default=[],
                      type=str,
                      help='metric key (name of the metric) to plot',
                      dest='metric_keys',
                      action='append')

    parser.add_option('-d',
                      '--decision_plot',
                      default=[],
                      type=str,
                      help='decision key (name of the decision) to plot',
                      dest='decision_keys',
                      action='append')

    options = parser.parse_args()[0]
    if options.dump_file_to_parse is None:
        print "No dump file to parse is set.\n"
        parser.print_help()
        exit()
    (metrics, decisions) = ParseAnaDump(options.dump_file_to_parse)
    metric_keys = options.metric_keys
    decision_keys = options.decision_keys
    plot_count = len(metric_keys) + len(decision_keys)
    if plot_count == 0:
        print "You have to set at least one metric or decision to plot.\n"
        parser.print_help()
        exit()
    plots = []
    if plot_count == 1:
        f, mp_plot = plt.subplots()
        plots.append(mp_plot)
    else:
        f, mp_plots = plt.subplots(plot_count, sharex=True)
        plots.extend(mp_plots.tolist())

    for key in metric_keys:
        plot = plots.pop()
        plot.grid(True)
        plot.set_title(key + " (metric)")
        plot.plot(metrics[key]['time'], metrics[key]['value'])
    for key in decision_keys:
        plot = plots.pop()
        plot.grid(True)
        plot.set_title(key + " (decision)")
        plot.plot(decisions[key]['time'], decisions[key]['value'])
    f.subplots_adjust(hspace=0.3)
    plt.show()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Shows boxplots of given score for different values of selected
parameters. Can be used to compare scores by audioproc_f flag.

Usage: apm_quality_assessment_boxplot.py -o /path/to/output
                                         -v polqa
                                         -n /path/to/dir/with/apm_configs
                                         -z audioproc_f_arg1 [arg2 ...]

Arguments --config_names, --render_names, --echo_simulator_names,
--test_data_generators, --eval_scores can be used to filter the data
used for plotting.
"""

import collections
import logging
import matplotlib.pyplot as plt
import os

import quality_assessment.data_access as data_access
import quality_assessment.collect_data as collect_data


def InstanceArgumentsParser():
    """Arguments parser factory.
  """
    parser = collect_data.InstanceArgumentsParser()
    parser.description = (
        'Shows boxplot of given score for different values of selected'
        'parameters. Can be used to compare scores by audioproc_f flag')

    parser.add_argument('-v',
                        '--eval_score',
                        required=True,
                        help=('Score name for constructing boxplots'))

    parser.add_argument(
        '-n',
        '--config_dir',
        required=False,
        help=('path to the folder with the configuration files'),
        default='apm_configs')

    parser.add_argument('-z',
                        '--params_to_plot',
                        required=True,
                        nargs='+',
                        help=('audioproc_f parameter values'
                              'by which to group scores (no leading dash)'))

    return parser


def FilterScoresByParams(data_frame, filter_params, score_name, config_dir):
    """Filters data on the values of one or more parameters.

  Args:
    data_frame: pandas.DataFrame of all used input data.

    filter_params: each config of the input data is assumed to have
      exactly one parameter from `filter_params` defined. Every value
      of the parameters in `filter_params` is a key in the returned
      dict; the associated value is all cells of the data with that
      value of the parameter.

    score_name: Name of score which value is boxplotted. Currently cannot do
      more than one value.

    config_dir: path to dir with APM configs.

  Returns: dictionary, key is a param value, result is all scores for
    that param value (see `filter_params` for explanation).
  """
    results = collections.defaultdict(dict)
    config_names = data_frame['apm_config'].drop_duplicates().values.tolist()

    for config_name in config_names:
        config_json = data_access.AudioProcConfigFile.Load(
            os.path.join(config_dir, config_name + '.json'))
        data_with_config = data_frame[data_frame.apm_config == config_name]
        data_cell_scores = data_with_config[data_with_config.eval_score_name ==
                                            score_name]

        # Exactly one of `params_to_plot` must match:
        (matching_param, ) = [
            x for x in filter_params if '-' + x in config_json
        ]

        # Add scores for every track to the result.
        for capture_name in data_cell_scores.capture:
            result_score = float(data_cell_scores[data_cell_scores.capture ==
                                                  capture_name].score)
            config_dict = results[config_json['-' + matching_param]]
            if capture_name not in config_dict:
                config_dict[capture_name] = {}

            config_dict[capture_name][matching_param] = result_score

    return results


def _FlattenToScoresList(config_param_score_dict):
    """Extracts a list of scores from input data structure.

  Args:
    config_param_score_dict: of the form {'capture_name':
    {'param_name' : score_value,.. } ..}

  Returns: Plain list of all score value present in input data
    structure
  """
    result = []
    for capture_name in config_param_score_dict:
        result += list(config_param_score_dict[capture_name].values())
    return result


def main():
    # Init.
    # TODO(alessiob): INFO once debugged.
    logging.basicConfig(level=logging.DEBUG)
    parser = InstanceArgumentsParser()
    args = parser.parse_args()

    # Get the scores.
    src_path = collect_data.ConstructSrcPath(args)
    logging.debug(src_path)
    scores_data_frame = collect_data.FindScores(src_path, args)

    # Filter the data by `args.params_to_plot`
    scores_filtered = FilterScoresByParams(scores_data_frame,
                                           args.params_to_plot,
                                           args.eval_score, args.config_dir)

    data_list = sorted(scores_filtered.items())
    data_values = [_FlattenToScoresList(x) for (_, x) in data_list]
    data_labels = [x for (x, _) in data_list]

    _, axes = plt.subplots(nrows=1, ncols=1, figsize=(6, 6))
    axes.boxplot(data_values, labels=data_labels)
    axes.set_ylabel(args.eval_score)
    axes.set_xlabel('/'.join(args.params_to_plot))
    plt.show()


if __name__ == "__main__":
    main()



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Unit tests for the apm_quality_assessment module.
"""

import sys
import unittest

import mock

import apm_quality_assessment


class TestSimulationScript(unittest.TestCase):
    """Unit tests for the apm_quality_assessment module.
  """

    def testMain(self):
        # Exit with error code if no arguments are passed.
        with self.assertRaises(SystemExit) as cm, mock.patch.object(
                sys, 'argv', ['apm_quality_assessment.py']):
            apm_quality_assessment.main()
        self.assertGreater(cm.exception.code, 0)



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Data access utility functions and classes.
"""

import json
import os


def MakeDirectory(path):
    """Makes a directory recursively without rising exceptions if existing.

  Args:
    path: path to the directory to be created.
  """
    if os.path.exists(path):
        return
    os.makedirs(path)


class Metadata(object):
    """Data access class to save and load metadata.
  """

    def __init__(self):
        pass

    _GENERIC_METADATA_SUFFIX = '.mdata'
    _AUDIO_TEST_DATA_FILENAME = 'audio_test_data.json'

    @classmethod
    def LoadFileMetadata(cls, filepath):
        """Loads generic metadata linked to a file.

    Args:
      filepath: path to the metadata file to read.

    Returns:
      A dict.
    """
        with open(filepath + cls._GENERIC_METADATA_SUFFIX) as f:
            return json.load(f)

    @classmethod
    def SaveFileMetadata(cls, filepath, metadata):
        """Saves generic metadata linked to a file.

    Args:
      filepath: path to the metadata file to write.
      metadata: a dict.
    """
        with open(filepath + cls._GENERIC_METADATA_SUFFIX, 'w') as f:
            json.dump(metadata, f)

    @classmethod
    def LoadAudioTestDataPaths(cls, metadata_path):
        """Loads the input and the reference audio track paths.

    Args:
      metadata_path: path to the directory containing the metadata file.

    Returns:
      Tuple with the paths to the input and output audio tracks.
    """
        metadata_filepath = os.path.join(metadata_path,
                                         cls._AUDIO_TEST_DATA_FILENAME)
        with open(metadata_filepath) as f:
            return json.load(f)

    @classmethod
    def SaveAudioTestDataPaths(cls, output_path, **filepaths):
        """Saves the input and the reference audio track paths.

    Args:
      output_path: path to the directory containing the metadata file.

    Keyword Args:
      filepaths: collection of audio track file paths to save.
    """
        output_filepath = os.path.join(output_path,
                                       cls._AUDIO_TEST_DATA_FILENAME)
        with open(output_filepath, 'w') as f:
            json.dump(filepaths, f)


class AudioProcConfigFile(object):
    """Data access to load/save APM simulator argument lists.

  The arguments stored in the config files are used to control the APM flags.
  """

    def __init__(self):
        pass

    @classmethod
    def Load(cls, filepath):
        """Loads a configuration file for an APM simulator.

    Args:
      filepath: path to the configuration file.

    Returns:
      A dict containing the configuration.
    """
        with open(filepath) as f:
            return json.load(f)

    @classmethod
    def Save(cls, filepath, config):
        """Saves a configuration file for an APM simulator.

    Args:
      filepath: path to the configuration file.
      config: a dict containing the configuration.
    """
        with open(filepath, 'w') as f:
            json.dump(config, f)


class ScoreFile(object):
    """Data access class to save and load float scalar scores.
  """

    def __init__(self):
        pass

    @classmethod
    def Load(cls, filepath):
        """Loads a score from file.

    Args:
      filepath: path to the score file.

    Returns:
      A float encoding the score.
    """
        with open(filepath) as f:
            return float(f.readline().strip())

    @classmethod
    def Save(cls, filepath, score):
        """Saves a score into a file.

    Args:
      filepath: path to the score file.
      score: float encoding the score.
    """
        with open(filepath, 'w') as f:
            f.write('{0:f}\n'.format(score))



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""APM module simulator.
"""

import logging
import os

from . import annotations
from . import data_access
from . import echo_path_simulation
from . import echo_path_simulation_factory
from . import eval_scores
from . import exceptions
from . import input_mixer
from . import input_signal_creator
from . import signal_processing
from . import test_data_generation


class ApmModuleSimulator(object):
    """Audio processing module (APM) simulator class.
  """

    _TEST_DATA_GENERATOR_CLASSES = (
        test_data_generation.TestDataGenerator.REGISTERED_CLASSES)
    _EVAL_SCORE_WORKER_CLASSES = eval_scores.EvaluationScore.REGISTERED_CLASSES

    _PREFIX_APM_CONFIG = 'apmcfg-'
    _PREFIX_CAPTURE = 'capture-'
    _PREFIX_RENDER = 'render-'
    _PREFIX_ECHO_SIMULATOR = 'echosim-'
    _PREFIX_TEST_DATA_GEN = 'datagen-'
    _PREFIX_TEST_DATA_GEN_PARAMS = 'datagen_params-'
    _PREFIX_SCORE = 'score-'

    def __init__(self,
                 test_data_generator_factory,
                 evaluation_score_factory,
                 ap_wrapper,
                 evaluator,
                 external_vads=None):
        if external_vads is None:
            external_vads = {}
        self._test_data_generator_factory = test_data_generator_factory
        self._evaluation_score_factory = evaluation_score_factory
        self._audioproc_wrapper = ap_wrapper
        self._evaluator = evaluator
        self._annotator = annotations.AudioAnnotationsExtractor(
            annotations.AudioAnnotationsExtractor.VadType.ENERGY_THRESHOLD
            | annotations.AudioAnnotationsExtractor.VadType.WEBRTC_COMMON_AUDIO
            | annotations.AudioAnnotationsExtractor.VadType.WEBRTC_APM,
            external_vads)

        # Init.
        self._test_data_generator_factory.SetOutputDirectoryPrefix(
            self._PREFIX_TEST_DATA_GEN_PARAMS)
        self._evaluation_score_factory.SetScoreFilenamePrefix(
            self._PREFIX_SCORE)

        # Properties for each run.
        self._base_output_path = None
        self._output_cache_path = None
        self._test_data_generators = None
        self._evaluation_score_workers = None
        self._config_filepaths = None
        self._capture_input_filepaths = None
        self._render_input_filepaths = None
        self._echo_path_simulator_class = None

    @classmethod
    def GetPrefixApmConfig(cls):
        return cls._PREFIX_APM_CONFIG

    @classmethod
    def GetPrefixCapture(cls):
        return cls._PREFIX_CAPTURE

    @classmethod
    def GetPrefixRender(cls):
        return cls._PREFIX_RENDER

    @classmethod
    def GetPrefixEchoSimulator(cls):
        return cls._PREFIX_ECHO_SIMULATOR

    @classmethod
    def GetPrefixTestDataGenerator(cls):
        return cls._PREFIX_TEST_DATA_GEN

    @classmethod
    def GetPrefixTestDataGeneratorParameters(cls):
        return cls._PREFIX_TEST_DATA_GEN_PARAMS

    @classmethod
    def GetPrefixScore(cls):
        return cls._PREFIX_SCORE

    def Run(self,
            config_filepaths,
            capture_input_filepaths,
            test_data_generator_names,
            eval_score_names,
            output_dir,
            render_input_filepaths=None,
            echo_path_simulator_name=(
                echo_path_simulation.NoEchoPathSimulator.NAME)):
        """Runs the APM simulation.

    Initializes paths and required instances, then runs all the simulations.
    The render input can be optionally added. If added, the number of capture
    input audio tracks and the number of render input audio tracks have to be
    equal. The two lists are used to form pairs of capture and render input.

    Args:
      config_filepaths: set of APM configuration files to test.
      capture_input_filepaths: set of capture input audio track files to test.
      test_data_generator_names: set of test data generator names to test.
      eval_score_names: set of evaluation score names to test.
      output_dir: base path to the output directory for wav files and outcomes.
      render_input_filepaths: set of render input audio track files to test.
      echo_path_simulator_name: name of the echo path simulator to use when
                                render input is provided.
    """
        assert render_input_filepaths is None or (
            len(capture_input_filepaths) == len(render_input_filepaths)), (
                'render input set size not matching input set size')
        assert render_input_filepaths is None or echo_path_simulator_name in (
            echo_path_simulation.EchoPathSimulator.REGISTERED_CLASSES), (
                'invalid echo path simulator')
        self._base_output_path = os.path.abspath(output_dir)

        # Output path used to cache the data shared across simulations.
        self._output_cache_path = os.path.join(self._base_output_path,
                                               '_cache')

        # Instance test data generators.
        self._test_data_generators = [
            self._test_data_generator_factory.GetInstance(
                test_data_generators_class=(
                    self._TEST_DATA_GENERATOR_CLASSES[name]))
            for name in (test_data_generator_names)
        ]

        # Instance evaluation score workers.
        self._evaluation_score_workers = [
            self._evaluation_score_factory.GetInstance(
                evaluation_score_class=self._EVAL_SCORE_WORKER_CLASSES[name])
            for (name) in eval_score_names
        ]

        # Set APM configuration file paths.
        self._config_filepaths = self._CreatePathsCollection(config_filepaths)

        # Set probing signal file paths.
        if render_input_filepaths is None:
            # Capture input only.
            self._capture_input_filepaths = self._CreatePathsCollection(
                capture_input_filepaths)
            self._render_input_filepaths = None
        else:
            # Set both capture and render input signals.
            self._SetTestInputSignalFilePaths(capture_input_filepaths,
                                              render_input_filepaths)

        # Set the echo path simulator class.
        self._echo_path_simulator_class = (
            echo_path_simulation.EchoPathSimulator.
            REGISTERED_CLASSES[echo_path_simulator_name])

        self._SimulateAll()

    def _SimulateAll(self):
        """Runs all the simulations.

    Iterates over the combinations of APM configurations, probing signals, and
    test data generators. This method is mainly responsible for the creation of
    the cache and output directories required in order to call _Simulate().
    """
        without_render_input = self._render_input_filepaths is None

        # Try different APM config files.
        for config_name in self._config_filepaths:
            config_filepath = self._config_filepaths[config_name]

            # Try different capture-render pairs.
            for capture_input_name in self._capture_input_filepaths:
                # Output path for the capture signal annotations.
                capture_annotations_cache_path = os.path.join(
                    self._output_cache_path,
                    self._PREFIX_CAPTURE + capture_input_name)
                data_access.MakeDirectory(capture_annotations_cache_path)

                # Capture.
                capture_input_filepath = self._capture_input_filepaths[
                    capture_input_name]
                if not os.path.exists(capture_input_filepath):
                    # If the input signal file does not exist, try to create using the
                    # available input signal creators.
                    self._CreateInputSignal(capture_input_filepath)
                assert os.path.exists(capture_input_filepath)
                self._ExtractCaptureAnnotations(
                    capture_input_filepath, capture_annotations_cache_path)

                # Render and simulated echo path (optional).
                render_input_filepath = None if without_render_input else (
                    self._render_input_filepaths[capture_input_name])
                render_input_name = '(none)' if without_render_input else (
                    self._ExtractFileName(render_input_filepath))
                echo_path_simulator = (echo_path_simulation_factory.
                                       EchoPathSimulatorFactory.GetInstance(
                                           self._echo_path_simulator_class,
                                           render_input_filepath))

                # Try different test data generators.
                for test_data_generators in self._test_data_generators:
                    logging.info(
                        'APM config preset: <%s>, capture: <%s>, render: <%s>,'
                        'test data generator: <%s>,  echo simulator: <%s>',
                        config_name, capture_input_name, render_input_name,
                        test_data_generators.NAME, echo_path_simulator.NAME)

                    # Output path for the generated test data.
                    test_data_cache_path = os.path.join(
                        capture_annotations_cache_path,
                        self._PREFIX_TEST_DATA_GEN + test_data_generators.NAME)
                    data_access.MakeDirectory(test_data_cache_path)
                    logging.debug('test data cache path: <%s>',
                                  test_data_cache_path)

                    # Output path for the echo simulator and APM input mixer output.
                    echo_test_data_cache_path = os.path.join(
                        test_data_cache_path,
                        'echosim-{}'.format(echo_path_simulator.NAME))
                    data_access.MakeDirectory(echo_test_data_cache_path)
                    logging.debug('echo test data cache path: <%s>',
                                  echo_test_data_cache_path)

                    # Full output path.
                    output_path = os.path.join(
                        self._base_output_path,
                        self._PREFIX_APM_CONFIG + config_name,
                        self._PREFIX_CAPTURE + capture_input_name,
                        self._PREFIX_RENDER + render_input_name,
                        self._PREFIX_ECHO_SIMULATOR + echo_path_simulator.NAME,
                        self._PREFIX_TEST_DATA_GEN + test_data_generators.NAME)
                    data_access.MakeDirectory(output_path)
                    logging.debug('output path: <%s>', output_path)

                    self._Simulate(test_data_generators,
                                   capture_input_filepath,
                                   render_input_filepath, test_data_cache_path,
                                   echo_test_data_cache_path, output_path,
                                   config_filepath, echo_path_simulator)

    @staticmethod
    def _CreateInputSignal(input_signal_filepath):
        """Creates a missing input signal file.

    The file name is parsed to extract input signal creator and params. If a
    creator is matched and the parameters are valid, a new signal is generated
    and written in `input_signal_filepath`.

    Args:
      input_signal_filepath: Path to the input signal audio file to write.

    Raises:
      InputSignalCreatorException
    """
        filename = os.path.splitext(
            os.path.split(input_signal_filepath)[-1])[0]
        filename_parts = filename.split('-')

        if len(filename_parts) < 2:
            raise exceptions.InputSignalCreatorException(
                'Cannot parse input signal file name')

        signal, metadata = input_signal_creator.InputSignalCreator.Create(
            filename_parts[0], filename_parts[1].split('_'))

        signal_processing.SignalProcessingUtils.SaveWav(
            input_signal_filepath, signal)
        data_access.Metadata.SaveFileMetadata(input_signal_filepath, metadata)

    def _ExtractCaptureAnnotations(self,
                                   input_filepath,
                                   output_path,
                                   annotation_name=""):
        self._annotator.Extract(input_filepath)
        self._annotator.Save(output_path, annotation_name)

    def _Simulate(self, test_data_generators, clean_capture_input_filepath,
                  render_input_filepath, test_data_cache_path,
                  echo_test_data_cache_path, output_path, config_filepath,
                  echo_path_simulator):
        """Runs a single set of simulation.

    Simulates a given combination of APM configuration, probing signal, and
    test data generator. It iterates over the test data generator
    internal configurations.

    Args:
      test_data_generators: TestDataGenerator instance.
      clean_capture_input_filepath: capture input audio track file to be
                                    processed by a test data generator and
                                    not affected by echo.
      render_input_filepath: render input audio track file to test.
      test_data_cache_path: path for the generated test audio track files.
      echo_test_data_cache_path: path for the echo simulator.
      output_path: base output path for the test data generator.
      config_filepath: APM configuration file to test.
      echo_path_simulator: EchoPathSimulator instance.
    """
        # Generate pairs of noisy input and reference signal files.
        test_data_generators.Generate(
            input_signal_filepath=clean_capture_input_filepath,
            test_data_cache_path=test_data_cache_path,
            base_output_path=output_path)

        # Extract metadata linked to the clean input file (if any).
        apm_input_metadata = None
        try:
            apm_input_metadata = data_access.Metadata.LoadFileMetadata(
                clean_capture_input_filepath)
        except IOError as e:
            apm_input_metadata = {}
        apm_input_metadata['test_data_gen_name'] = test_data_generators.NAME
        apm_input_metadata['test_data_gen_config'] = None

        # For each test data pair, simulate a call and evaluate.
        for config_name in test_data_generators.config_names:
            logging.info(' - test data generator config: <%s>', config_name)
            apm_input_metadata['test_data_gen_config'] = config_name

            # Paths to the test data generator output.
            # Note that the reference signal does not depend on the render input
            # which is optional.
            noisy_capture_input_filepath = (
                test_data_generators.noisy_signal_filepaths[config_name])
            reference_signal_filepath = (
                test_data_generators.reference_signal_filepaths[config_name])

            # Output path for the evaluation (e.g., APM output file).
            evaluation_output_path = test_data_generators.apm_output_paths[
                config_name]

            # Paths to the APM input signals.
            echo_path_filepath = echo_path_simulator.Simulate(
                echo_test_data_cache_path)
            apm_input_filepath = input_mixer.ApmInputMixer.Mix(
                echo_test_data_cache_path, noisy_capture_input_filepath,
                echo_path_filepath)

            # Extract annotations for the APM input mix.
            apm_input_basepath, apm_input_filename = os.path.split(
                apm_input_filepath)
            self._ExtractCaptureAnnotations(
                apm_input_filepath, apm_input_basepath,
                os.path.splitext(apm_input_filename)[0] + '-')

            # Simulate a call using APM.
            self._audioproc_wrapper.Run(
                config_filepath=config_filepath,
                capture_input_filepath=apm_input_filepath,
                render_input_filepath=render_input_filepath,
                output_path=evaluation_output_path)

            try:
                # Evaluate.
                self._evaluator.Run(
                    evaluation_score_workers=self._evaluation_score_workers,
                    apm_input_metadata=apm_input_metadata,
                    apm_output_filepath=self._audioproc_wrapper.
                    output_filepath,
                    reference_input_filepath=reference_signal_filepath,
                    render_input_filepath=render_input_filepath,
                    output_path=evaluation_output_path,
                )

                # Save simulation metadata.
                data_access.Metadata.SaveAudioTestDataPaths(
                    output_path=evaluation_output_path,
                    clean_capture_input_filepath=clean_capture_input_filepath,
                    echo_free_capture_filepath=noisy_capture_input_filepath,
                    echo_filepath=echo_path_filepath,
                    render_filepath=render_input_filepath,
                    capture_filepath=apm_input_filepath,
                    apm_output_filepath=self._audioproc_wrapper.
                    output_filepath,
                    apm_reference_filepath=reference_signal_filepath,
                    apm_config_filepath=config_filepath,
                )
            except exceptions.EvaluationScoreException as e:
                logging.warning('the evaluation failed: %s', e.message)
                continue

    def _SetTestInputSignalFilePaths(self, capture_input_filepaths,
                                     render_input_filepaths):
        """Sets input and render input file paths collections.

    Pairs the input and render input files by storing the file paths into two
    collections. The key is the file name of the input file.

    Args:
      capture_input_filepaths: list of file paths.
      render_input_filepaths: list of file paths.
    """
        self._capture_input_filepaths = {}
        self._render_input_filepaths = {}
        assert len(capture_input_filepaths) == len(render_input_filepaths)
        for capture_input_filepath, render_input_filepath in zip(
                capture_input_filepaths, render_input_filepaths):
            name = self._ExtractFileName(capture_input_filepath)
            self._capture_input_filepaths[name] = os.path.abspath(
                capture_input_filepath)
            self._render_input_filepaths[name] = os.path.abspath(
                render_input_filepath)

    @classmethod
    def _CreatePathsCollection(cls, filepaths):
        """Creates a collection of file paths.

    Given a list of file paths, makes a collection with one item for each file
    path. The value is absolute path, the key is the file name without
    extenstion.

    Args:
      filepaths: list of file paths.

    Returns:
      A dict.
    """
        filepaths_collection = {}
        for filepath in filepaths:
            name = cls._ExtractFileName(filepath)
            filepaths_collection[name] = os.path.abspath(filepath)
        return filepaths_collection

    @classmethod
    def _ExtractFileName(cls, filepath):
        return os.path.splitext(os.path.split(filepath)[-1])[0]



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Echo path simulation factory module.
"""

import numpy as np

from . import echo_path_simulation


class EchoPathSimulatorFactory(object):

    # TODO(alessiob): Replace 20 ms delay (at 48 kHz sample rate) with a more
    # realistic impulse response.
    _LINEAR_ECHO_IMPULSE_RESPONSE = np.array([0.0] * (20 * 48) + [0.15])

    def __init__(self):
        pass

    @classmethod
    def GetInstance(cls, echo_path_simulator_class, render_input_filepath):
        """Creates an EchoPathSimulator instance given a class object.

    Args:
      echo_path_simulator_class: EchoPathSimulator class object (not an
                                 instance).
      render_input_filepath: Path to the render audio track file.

    Returns:
      An EchoPathSimulator instance.
    """
        assert render_input_filepath is not None or (
            echo_path_simulator_class ==
            echo_path_simulation.NoEchoPathSimulator)

        if echo_path_simulator_class == echo_path_simulation.NoEchoPathSimulator:
            return echo_path_simulation.NoEchoPathSimulator()
        elif echo_path_simulator_class == (
                echo_path_simulation.LinearEchoPathSimulator):
            return echo_path_simulation.LinearEchoPathSimulator(
                render_input_filepath, cls._LINEAR_ECHO_IMPULSE_RESPONSE)
        else:
            return echo_path_simulator_class(render_input_filepath)



#!/usr/bin/env python
# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Finds the APM configuration that maximizes a provided metric by
parsing the output generated apm_quality_assessment.py.
"""

from __future__ import division

import collections
import logging
import os

import quality_assessment.data_access as data_access
import quality_assessment.collect_data as collect_data


def _InstanceArgumentsParser():
    """Arguments parser factory. Extends the arguments from 'collect_data'
  with a few extra for selecting what parameters to optimize for.
  """
    parser = collect_data.InstanceArgumentsParser()
    parser.description = (
        'Rudimentary optimization of a function over different parameter'
        'combinations.')

    parser.add_argument(
        '-n',
        '--config_dir',
        required=False,
        help=('path to the folder with the configuration files'),
        default='apm_configs')

    parser.add_argument('-p',
                        '--params',
                        required=True,
                        nargs='+',
                        help=('parameters to parse from the config files in'
                              'config_dir'))

    parser.add_argument(
        '-z',
        '--params_not_to_optimize',
        required=False,
        nargs='+',
        default=[],
        help=('parameters from `params` not to be optimized for'))

    return parser


def _ConfigurationAndScores(data_frame, params, params_not_to_optimize,
                            config_dir):
    """Returns a list of all configurations and scores.

  Args:
    data_frame: A pandas data frame with the scores and config name
                returned by _FindScores.
    params: The parameter names to parse from configs the config
            directory

    params_not_to_optimize: The parameter names which shouldn't affect
                            the optimal parameter
                            selection. E.g., fixed settings and not
                            tunable parameters.

    config_dir: Path to folder with config files.

  Returns:
    Dictionary of the form
    {param_combination: [{params: {param1: value1, ...},
                          scores: {score1: value1, ...}}]}.

    The key `param_combination` runs over all parameter combinations
    of the parameters in `params` and not in
    `params_not_to_optimize`. A corresponding value is a list of all
    param combinations for params in `params_not_to_optimize` and
    their scores.
  """
    results = collections.defaultdict(list)
    config_names = data_frame['apm_config'].drop_duplicates().values.tolist()
    score_names = data_frame['eval_score_name'].drop_duplicates(
    ).values.tolist()

    # Normalize the scores
    normalization_constants = {}
    for score_name in score_names:
        scores = data_frame[data_frame.eval_score_name == score_name].score
        normalization_constants[score_name] = max(scores)

    params_to_optimize = [p for p in params if p not in params_not_to_optimize]
    param_combination = collections.namedtuple("ParamCombination",
                                               params_to_optimize)

    for config_name in config_names:
        config_json = data_access.AudioProcConfigFile.Load(
            os.path.join(config_dir, config_name + ".json"))
        scores = {}
        data_cell = data_frame[data_frame.apm_config == config_name]
        for score_name in score_names:
            data_cell_scores = data_cell[data_cell.eval_score_name ==
                                         score_name].score
            scores[score_name] = sum(data_cell_scores) / len(data_cell_scores)
            scores[score_name] /= normalization_constants[score_name]

        result = {'scores': scores, 'params': {}}
        config_optimize_params = {}
        for param in params:
            if param in params_to_optimize:
                config_optimize_params[param] = config_json['-' + param]
            else:
                result['params'][param] = config_json['-' + param]

        current_param_combination = param_combination(**config_optimize_params)
        results[current_param_combination].append(result)
    return results


def _FindOptimalParameter(configs_and_scores, score_weighting):
    """Finds the config producing the maximal score.

  Args:
    configs_and_scores: structure of the form returned by
                        _ConfigurationAndScores

    score_weighting: a function to weight together all score values of
                     the form [{params: {param1: value1, ...}, scores:
                                {score1: value1, ...}}] into a numeric
                     value
  Returns:
    the config that has the largest values of `score_weighting` applied
    to its scores.
  """

    min_score = float('+inf')
    best_params = None
    for config in configs_and_scores:
        scores_and_params = configs_and_scores[config]
        current_score = score_weighting(scores_and_params)
        if current_score < min_score:
            min_score = current_score
            best_params = config
            logging.debug("Score: %f", current_score)
            logging.debug("Config: %s", str(config))
    return best_params


def _ExampleWeighting(scores_and_configs):
    """Example argument to `_FindOptimalParameter`
  Args:
    scores_and_configs: a list of configs and scores, in the form
                        described in _FindOptimalParameter
  Returns:
    numeric value, the sum of all scores
  """
    res = 0
    for score_config in scores_and_configs:
        res += sum(score_config['scores'].values())
    return res


def main():
    # Init.
    # TODO(alessiob): INFO once debugged.
    logging.basicConfig(level=logging.DEBUG)
    parser = _InstanceArgumentsParser()
    args = parser.parse_args()

    # Get the scores.
    src_path = collect_data.ConstructSrcPath(args)
    logging.debug('Src path <%s>', src_path)
    scores_data_frame = collect_data.FindScores(src_path, args)
    all_scores = _ConfigurationAndScores(scores_data_frame, args.params,
                                         args.params_not_to_optimize,
                                         args.config_dir)

    opt_param = _FindOptimalParameter(all_scores, _ExampleWeighting)

    logging.info('Optimal parameter combination: <%s>', opt_param)
    logging.info('It\'s score values: <%s>', all_scores[opt_param])


if __name__ == "__main__":
    main()



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Input signal creator module.
"""

from . import exceptions
from . import signal_processing


class InputSignalCreator(object):
    """Input signal creator class.
  """

    @classmethod
    def Create(cls, name, raw_params):
        """Creates a input signal and its metadata.

    Args:
      name: Input signal creator name.
      raw_params: Tuple of parameters to pass to the specific signal creator.

    Returns:
      (AudioSegment, dict) tuple.
    """
        try:
            signal = {}
            params = {}

            if name == 'pure_tone':
                params['frequency'] = float(raw_params[0])
                params['duration'] = int(raw_params[1])
                signal = cls._CreatePureTone(params['frequency'],
                                             params['duration'])
            else:
                raise exceptions.InputSignalCreatorException(
                    'Invalid input signal creator name')

            # Complete metadata.
            params['signal'] = name

            return signal, params
        except (TypeError, AssertionError) as e:
            raise exceptions.InputSignalCreatorException(
                'Invalid signal creator parameters: {}'.format(e))

    @classmethod
    def _CreatePureTone(cls, frequency, duration):
        """
    Generates a pure tone at 48000 Hz.

    Args:
      frequency: Float in (0-24000] (Hz).
      duration: Integer (milliseconds).

    Returns:
      AudioSegment instance.
    """
        assert 0 < frequency <= 24000
        assert duration > 0
        template = signal_processing.SignalProcessingUtils.GenerateSilence(
            duration)
        return signal_processing.SignalProcessingUtils.GeneratePureTone(
            template, frequency)



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.



# Copyright 2017 The LibYuv Project Authors. All rights reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS. All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

# Runs PRESUBMIT.py in py3 mode by git cl presubmit.
USE_PYTHON3 = True

def _CommonChecks(input_api, output_api):
  """Checks common to both upload and commit."""
  results = []
  results.extend(input_api.canned_checks.RunPylint(input_api, output_api,
      files_to_skip=(r'^base[\\\/].*\.py$',
                     r'^build[\\\/].*\.py$',
                     r'^buildtools[\\\/].*\.py$',
                     r'^ios[\\\/].*\.py$',
                     r'^out.*[\\\/].*\.py$',
                     r'^testing[\\\/].*\.py$',
                     r'^third_party[\\\/].*\.py$',
                     r'^tools[\\\/].*\.py$',
                     # TODO(kjellander): should arguably be checked.
                     r'^tools_libyuv[\\\/]valgrind[\\\/].*\.py$',
                     r'^xcodebuild.*[\\\/].*\.py$',),
      disabled_warnings=['F0401',  # Failed to import x
                         'E0611',  # No package y in x
                         'W0232',  # Class has no __init__ method
                        ],
      pylintrc='pylintrc',
      version='2.7'))
  return results


def CheckChangeOnUpload(input_api, output_api):
  results = []
  results.extend(_CommonChecks(input_api, output_api))
  results.extend(
      input_api.canned_checks.CheckGNFormatted(input_api, output_api))
  return results


def CheckChangeOnCommit(input_api, output_api):
  results = []
  results.extend(_CommonChecks(input_api, output_api))
  results.extend(input_api.canned_checks.CheckOwners(input_api, output_api))
  results.extend(input_api.canned_checks.CheckChangeWasUploaded(
      input_api, output_api))
  results.extend(input_api.canned_checks.CheckChangeHasDescription(
      input_api, output_api))
  return results



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Evaluation score abstract class and implementations.
"""

from __future__ import division
import logging
import os
import re
import subprocess
import sys

try:
    import numpy as np
except ImportError:
    logging.critical('Cannot import the third-party Python package numpy')
    sys.exit(1)

from . import data_access
from . import exceptions
from . import signal_processing


class EvaluationScore(object):

    NAME = None
    REGISTERED_CLASSES = {}

    def __init__(self, score_filename_prefix):
        self._score_filename_prefix = score_filename_prefix
        self._input_signal_metadata = None
        self._reference_signal = None
        self._reference_signal_filepath = None
        self._tested_signal = None
        self._tested_signal_filepath = None
        self._output_filepath = None
        self._score = None
        self._render_signal_filepath = None

    @classmethod
    def RegisterClass(cls, class_to_register):
        """Registers an EvaluationScore implementation.

    Decorator to automatically register the classes that extend EvaluationScore.
    Example usage:

    @EvaluationScore.RegisterClass
    class AudioLevelScore(EvaluationScore):
      pass
    """
        cls.REGISTERED_CLASSES[class_to_register.NAME] = class_to_register
        return class_to_register

    @property
    def output_filepath(self):
        return self._output_filepath

    @property
    def score(self):
        return self._score

    def SetInputSignalMetadata(self, metadata):
        """Sets input signal metadata.

    Args:
      metadata: dict instance.
    """
        self._input_signal_metadata = metadata

    def SetReferenceSignalFilepath(self, filepath):
        """Sets the path to the audio track used as reference signal.

    Args:
      filepath: path to the reference audio track.
    """
        self._reference_signal_filepath = filepath

    def SetTestedSignalFilepath(self, filepath):
        """Sets the path to the audio track used as test signal.

    Args:
      filepath: path to the test audio track.
    """
        self._tested_signal_filepath = filepath

    def SetRenderSignalFilepath(self, filepath):
        """Sets the path to the audio track used as render signal.

    Args:
      filepath: path to the test audio track.
    """
        self._render_signal_filepath = filepath

    def Run(self, output_path):
        """Extracts the score for the set test data pair.

    Args:
      output_path: path to the directory where the output is written.
    """
        self._output_filepath = os.path.join(
            output_path, self._score_filename_prefix + self.NAME + '.txt')
        try:
            # If the score has already been computed, load.
            self._LoadScore()
            logging.debug('score found and loaded')
        except IOError:
            # Compute the score.
            logging.debug('score not found, compute')
            self._Run(output_path)

    def _Run(self, output_path):
        # Abstract method.
        raise NotImplementedError()

    def _LoadReferenceSignal(self):
        assert self._reference_signal_filepath is not None
        self._reference_signal = signal_processing.SignalProcessingUtils.LoadWav(
            self._reference_signal_filepath)

    def _LoadTestedSignal(self):
        assert self._tested_signal_filepath is not None
        self._tested_signal = signal_processing.SignalProcessingUtils.LoadWav(
            self._tested_signal_filepath)

    def _LoadScore(self):
        return data_access.ScoreFile.Load(self._output_filepath)

    def _SaveScore(self):
        return data_access.ScoreFile.Save(self._output_filepath, self._score)


@EvaluationScore.RegisterClass
class AudioLevelPeakScore(EvaluationScore):
    """Peak audio level score.

  Defined as the difference between the peak audio level of the tested and
  the reference signals.

  Unit: dB
  Ideal: 0 dB
  Worst case: +/-inf dB
  """

    NAME = 'audio_level_peak'

    def __init__(self, score_filename_prefix):
        EvaluationScore.__init__(self, score_filename_prefix)

    def _Run(self, output_path):
        self._LoadReferenceSignal()
        self._LoadTestedSignal()
        self._score = self._tested_signal.dBFS - self._reference_signal.dBFS
        self._SaveScore()


@EvaluationScore.RegisterClass
class MeanAudioLevelScore(EvaluationScore):
    """Mean audio level score.

  Defined as the difference between the mean audio level of the tested and
  the reference signals.

  Unit: dB
  Ideal: 0 dB
  Worst case: +/-inf dB
  """

    NAME = 'audio_level_mean'

    def __init__(self, score_filename_prefix):
        EvaluationScore.__init__(self, score_filename_prefix)

    def _Run(self, output_path):
        self._LoadReferenceSignal()
        self._LoadTestedSignal()

        dbfs_diffs_sum = 0.0
        seconds = min(len(self._tested_signal), len(
            self._reference_signal)) // 1000
        for t in range(seconds):
            t0 = t * seconds
            t1 = t0 + seconds
            dbfs_diffs_sum += (self._tested_signal[t0:t1].dBFS -
                               self._reference_signal[t0:t1].dBFS)
        self._score = dbfs_diffs_sum / float(seconds)
        self._SaveScore()


@EvaluationScore.RegisterClass
class EchoMetric(EvaluationScore):
    """Echo score.

  Proportion of detected echo.

  Unit: ratio
  Ideal: 0
  Worst case: 1
  """

    NAME = 'echo_metric'

    def __init__(self, score_filename_prefix, echo_detector_bin_filepath):
        EvaluationScore.__init__(self, score_filename_prefix)

        # POLQA binary file path.
        self._echo_detector_bin_filepath = echo_detector_bin_filepath
        if not os.path.exists(self._echo_detector_bin_filepath):
            logging.error('cannot find EchoMetric tool binary file')
            raise exceptions.FileNotFoundError()

        self._echo_detector_bin_path, _ = os.path.split(
            self._echo_detector_bin_filepath)

    def _Run(self, output_path):
        echo_detector_out_filepath = os.path.join(output_path,
                                                  'echo_detector.out')
        if os.path.exists(echo_detector_out_filepath):
            os.unlink(echo_detector_out_filepath)

        logging.debug("Render signal filepath: %s",
                      self._render_signal_filepath)
        if not os.path.exists(self._render_signal_filepath):
            logging.error(
                "Render input required for evaluating the echo metric.")

        args = [
            self._echo_detector_bin_filepath, '--output_file',
            echo_detector_out_filepath, '--', '-i',
            self._tested_signal_filepath, '-ri', self._render_signal_filepath
        ]
        logging.debug(' '.join(args))
        subprocess.call(args, cwd=self._echo_detector_bin_path)

        # Parse Echo detector tool output and extract the score.
        self._score = self._ParseOutputFile(echo_detector_out_filepath)
        self._SaveScore()

    @classmethod
    def _ParseOutputFile(cls, echo_metric_file_path):
        """
    Parses the POLQA tool output formatted as a table ('-t' option).

    Args:
      polqa_out_filepath: path to the POLQA tool output file.

    Returns:
      The score as a number in [0, 1].
    """
        with open(echo_metric_file_path) as f:
            return float(f.read())


@EvaluationScore.RegisterClass
class PolqaScore(EvaluationScore):
    """POLQA score.

  See http://www.polqa.info/.

  Unit: MOS
  Ideal: 4.5
  Worst case: 1.0
  """

    NAME = 'polqa'

    def __init__(self, score_filename_prefix, polqa_bin_filepath):
        EvaluationScore.__init__(self, score_filename_prefix)

        # POLQA binary file path.
        self._polqa_bin_filepath = polqa_bin_filepath
        if not os.path.exists(self._polqa_bin_filepath):
            logging.error('cannot find POLQA tool binary file')
            raise exceptions.FileNotFoundError()

        # Path to the POLQA directory with binary and license files.
        self._polqa_tool_path, _ = os.path.split(self._polqa_bin_filepath)

    def _Run(self, output_path):
        polqa_out_filepath = os.path.join(output_path, 'polqa.out')
        if os.path.exists(polqa_out_filepath):
            os.unlink(polqa_out_filepath)

        args = [
            self._polqa_bin_filepath,
            '-t',
            '-q',
            '-Overwrite',
            '-Ref',
            self._reference_signal_filepath,
            '-Test',
            self._tested_signal_filepath,
            '-LC',
            'NB',
            '-Out',
            polqa_out_filepath,
        ]
        logging.debug(' '.join(args))
        subprocess.call(args, cwd=self._polqa_tool_path)

        # Parse POLQA tool output and extract the score.
        polqa_output = self._ParseOutputFile(polqa_out_filepath)
        self._score = float(polqa_output['PolqaScore'])

        self._SaveScore()

    @classmethod
    def _ParseOutputFile(cls, polqa_out_filepath):
        """
    Parses the POLQA tool output formatted as a table ('-t' option).

    Args:
      polqa_out_filepath: path to the POLQA tool output file.

    Returns:
      A dict.
    """
        data = []
        with open(polqa_out_filepath) as f:
            for line in f:
                line = line.strip()
                if len(line) == 0 or line.startswith('*'):
                    # Ignore comments.
                    continue
                # Read fields.
                data.append(re.split(r'\t+', line))

        # Two rows expected (header and values).
        assert len(data) == 2, 'Cannot parse POLQA output'
        number_of_fields = len(data[0])
        assert number_of_fields == len(data[1])

        # Build and return a dictionary with field names (header) as keys and the
        # corresponding field values as values.
        return {
            data[0][index]: data[1][index]
            for index in range(number_of_fields)
        }


@EvaluationScore.RegisterClass
class TotalHarmonicDistorsionScore(EvaluationScore):
    """Total harmonic distorsion plus noise score.

  Total harmonic distorsion plus noise score.
  See "https://en.wikipedia.org/wiki/Total_harmonic_distortion#THD.2BN".

  Unit: -.
  Ideal: 0.
  Worst case: +inf
  """

    NAME = 'thd'

    def __init__(self, score_filename_prefix):
        EvaluationScore.__init__(self, score_filename_prefix)
        self._input_frequency = None

    def _Run(self, output_path):
        self._CheckInputSignal()

        self._LoadTestedSignal()
        if self._tested_signal.channels != 1:
            raise exceptions.EvaluationScoreException(
                'unsupported number of channels')
        samples = signal_processing.SignalProcessingUtils.AudioSegmentToRawData(
            self._tested_signal)

        # Init.
        num_samples = len(samples)
        duration = len(self._tested_signal) / 1000.0
        scaling = 2.0 / num_samples
        max_freq = self._tested_signal.frame_rate / 2
        f0_freq = float(self._input_frequency)
        t = np.linspace(0, duration, num_samples)

        # Analyze harmonics.
        b_terms = []
        n = 1
        while f0_freq * n < max_freq:
            x_n = np.sum(
                samples * np.sin(2.0 * np.pi * n * f0_freq * t)) * scaling
            y_n = np.sum(
                samples * np.cos(2.0 * np.pi * n * f0_freq * t)) * scaling
            b_terms.append(np.sqrt(x_n**2 + y_n**2))
            n += 1

        output_without_fundamental = samples - b_terms[0] * np.sin(
            2.0 * np.pi * f0_freq * t)
        distortion_and_noise = np.sqrt(
            np.sum(output_without_fundamental**2) * np.pi * scaling)

        # TODO(alessiob): Fix or remove if not needed.
        # thd = np.sqrt(np.sum(b_terms[1:]**2)) / b_terms[0]

        # TODO(alessiob): Check the range of `thd_plus_noise` and update the class
        # docstring above if accordingly.
        thd_plus_noise = distortion_and_noise / b_terms[0]

        self._score = thd_plus_noise
        self._SaveScore()

    def _CheckInputSignal(self):
        # Check input signal and get properties.
        try:
            if self._input_signal_metadata['signal'] != 'pure_tone':
                raise exceptions.EvaluationScoreException(
                    'The THD score requires a pure tone as input signal')
            self._input_frequency = self._input_signal_metadata['frequency']
            if self._input_signal_metadata[
                    'test_data_gen_name'] != 'identity' or (
                        self._input_signal_metadata['test_data_gen_config'] !=
                        'default'):
                raise exceptions.EvaluationScoreException(
                    'The THD score cannot be used with any test data generator other '
                    'than "identity"')
        except TypeError:
            raise exceptions.EvaluationScoreException(
                'The THD score requires an input signal with associated metadata'
            )
        except KeyError:
            raise exceptions.EvaluationScoreException(
                'Invalid input signal metadata to compute the THD score')



#!/usr/bin/python
# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
import argparse
import numpy as np


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True)
    parser.add_argument('-o', required=True)

    args = parser.parse_args()

    array = np.arange(100, dtype=np.float32)
    array.tofile(open(args.o, 'w'))


if __name__ == '__main__':
    main()



#!/usr/bin/env vpython3

# Copyright 2017 The LibYuv Project Authors. All rights reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS. All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

# This is a modified copy of the script in
# https://webrtc.googlesource.com/src/+/master/tools_webrtc/autoroller/roll_deps.py
# customized for libyuv.

"""Script to automatically roll dependencies in the libyuv DEPS file."""

import argparse
import base64
import collections
import logging
import os
import re
import subprocess
import sys
import urllib.request


# Skip these dependencies (list without solution name prefix).
DONT_AUTOROLL_THESE = [
  'src/third_party/gflags/src',
]

LIBYUV_URL = 'https://chromium.googlesource.com/libyuv/libyuv'
CHROMIUM_SRC_URL = 'https://chromium.googlesource.com/chromium/src'
CHROMIUM_COMMIT_TEMPLATE = CHROMIUM_SRC_URL + '/+/%s'
CHROMIUM_LOG_TEMPLATE = CHROMIUM_SRC_URL + '/+log/%s'
CHROMIUM_FILE_TEMPLATE = CHROMIUM_SRC_URL + '/+/%s/%s'

COMMIT_POSITION_RE = re.compile('^Cr-Commit-Position: .*#([0-9]+).*$')
CLANG_REVISION_RE = re.compile(r'^CLANG_REVISION = \'([0-9a-z-]+)\'$')
ROLL_BRANCH_NAME = 'roll_chromium_revision'

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
CHECKOUT_SRC_DIR = os.path.realpath(os.path.join(SCRIPT_DIR, os.pardir,
                                                 os.pardir))
CHECKOUT_ROOT_DIR = os.path.realpath(os.path.join(CHECKOUT_SRC_DIR, os.pardir))

sys.path.append(os.path.join(CHECKOUT_SRC_DIR, 'build'))
import find_depot_tools  # pylint: disable=wrong-import-position
find_depot_tools.add_depot_tools_to_path()

CLANG_UPDATE_SCRIPT_URL_PATH = 'tools/clang/scripts/update.py'
CLANG_UPDATE_SCRIPT_LOCAL_PATH = os.path.join(CHECKOUT_SRC_DIR, 'tools',
                                              'clang', 'scripts', 'update.py')

DepsEntry = collections.namedtuple('DepsEntry', 'path url revision')
ChangedDep = collections.namedtuple('ChangedDep',
                                    'path url current_rev new_rev')

class RollError(Exception):
  pass


def VarLookup(local_scope):
  return lambda var_name: local_scope['vars'][var_name]


def ParseDepsDict(deps_content):
  local_scope = {}
  global_scope = {
    'Var': VarLookup(local_scope),
    'Str': lambda s: s,
    'deps_os': {},
  }
  exec(deps_content, global_scope, local_scope)
  return local_scope


def ParseLocalDepsFile(filename):
  with open(filename, 'rb') as f:
    deps_content = f.read().decode('utf-8')
  return ParseDepsDict(deps_content)


def ParseRemoteCrDepsFile(revision):
  deps_content = ReadRemoteCrFile('DEPS', revision)
  return ParseDepsDict(deps_content)


def ParseCommitPosition(commit_message):
  for line in reversed(commit_message.splitlines()):
    m = COMMIT_POSITION_RE.match(line.strip())
    if m:
      return int(m.group(1))
  logging.error('Failed to parse commit position id from:\n%s\n',
                commit_message)
  sys.exit(-1)


def _RunCommand(command, working_dir=None, ignore_exit_code=False,
                extra_env=None, input_data=None):
  """Runs a command and returns the output from that command.

  If the command fails (exit code != 0), the function will exit the process.

  Returns:
    A tuple containing the stdout and stderr outputs as strings.
  """
  working_dir = working_dir or CHECKOUT_SRC_DIR
  logging.debug('CMD: %s CWD: %s', ' '.join(command), working_dir)
  env = os.environ.copy()
  if extra_env:
    assert all(isinstance(value, str) for value in extra_env.values())
    logging.debug('extra env: %s', extra_env)
    env.update(extra_env)
  p = subprocess.Popen(command,
                       stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       stderr=subprocess.PIPE,
                       env=env,
                       cwd=working_dir,
                       universal_newlines=True)
  std_output, err_output = p.communicate(input_data)
  p.stdout.close()
  p.stderr.close()
  if not ignore_exit_code and p.returncode != 0:
    logging.error('Command failed: %s\n'
                  'stdout:\n%s\n'
                  'stderr:\n%s\n', ' '.join(command), std_output, err_output)
    sys.exit(p.returncode)
  return std_output, err_output


def _GetBranches():
  """Returns a tuple of active,branches.

  The 'active' is the name of the currently active branch and 'branches' is a
  list of all branches.
  """
  lines = _RunCommand(['git', 'branch'])[0].split('\n')
  branches = []
  active = ''
  for line in lines:
    if '*' in line:
      # The assumption is that the first char will always be the '*'.
      active = line[1:].strip()
      branches.append(active)
    else:
      branch = line.strip()
      if branch:
        branches.append(branch)
  return active, branches


def _ReadGitilesContent(url):
  # Download and decode BASE64 content until
  # https://code.google.com/p/gitiles/issues/detail?id=7 is fixed.
  base64_content = ReadUrlContent(url + '?format=TEXT')
  return base64.b64decode(base64_content[0]).decode('utf-8')


def ReadRemoteCrFile(path_below_src, revision):
  """Reads a remote Chromium file of a specific revision. Returns a string."""
  return _ReadGitilesContent(CHROMIUM_FILE_TEMPLATE % (revision,
                                                       path_below_src))


def ReadRemoteCrCommit(revision):
  """Reads a remote Chromium commit message. Returns a string."""
  return _ReadGitilesContent(CHROMIUM_COMMIT_TEMPLATE % revision)


def ReadUrlContent(url):
  """Connect to a remote host and read the contents. Returns a list of lines."""
  conn = urllib.request.urlopen(url)
  try:
    return conn.readlines()
  except IOError as e:
    logging.exception('Error connecting to %s. Error: %s', url, e)
    raise
  finally:
    conn.close()


def GetMatchingDepsEntries(depsentry_dict, dir_path):
  """Gets all deps entries matching the provided path.

  This list may contain more than one DepsEntry object.
  Example: dir_path='src/testing' would give results containing both
  'src/testing/gtest' and 'src/testing/gmock' deps entries for Chromium's DEPS.
  Example 2: dir_path='src/build' should return 'src/build' but not
  'src/buildtools'.

  Returns:
    A list of DepsEntry objects.
  """
  result = []
  for path, depsentry in depsentry_dict.items():
    if path == dir_path:
      result.append(depsentry)
    else:
      parts = path.split('/')
      if all(part == parts[i]
             for i, part in enumerate(dir_path.split('/'))):
        result.append(depsentry)
  return result

def BuildDepsentryDict(deps_dict):
  """Builds a dict of paths to DepsEntry objects from a raw deps dict."""
  result = {}

  def AddDepsEntries(deps_subdict):
    for path, deps_url_spec in deps_subdict.items():
      if isinstance(deps_url_spec, dict):
        if deps_url_spec.get('dep_type') == 'cipd':
          continue
        deps_url = deps_url_spec['url']
      else:
        deps_url = deps_url_spec
      if not path in result:
        url, revision = deps_url.split('@') if deps_url else (None, None)
        result[path] = DepsEntry(path, url, revision)

  AddDepsEntries(deps_dict['deps'])
  for deps_os in ['win', 'mac', 'linux', 'android', 'ios', 'unix']:
    AddDepsEntries(deps_dict.get('deps_os', {}).get(deps_os, {}))
  return result


def CalculateChangedDeps(libyuv_deps, new_cr_deps):
  """
  Calculate changed deps entries based on entries defined in the libyuv DEPS
  file:
     - If a shared dependency with the Chromium DEPS file: roll it to the same
       revision as Chromium (i.e. entry in the new_cr_deps dict)
     - If it's a Chromium sub-directory, roll it to the HEAD revision (notice
       this means it may be ahead of the chromium_revision, but generally these
       should be close).
     - If it's another DEPS entry (not shared with Chromium), roll it to HEAD
       unless it's configured to be skipped.

  Returns:
    A list of ChangedDep objects representing the changed deps.
  """
  result = []
  libyuv_entries = BuildDepsentryDict(libyuv_deps)
  new_cr_entries = BuildDepsentryDict(new_cr_deps)
  for path, libyuv_deps_entry in libyuv_entries.items():
    if path in DONT_AUTOROLL_THESE:
      continue
    cr_deps_entry = new_cr_entries.get(path)
    if cr_deps_entry:
      # Use the revision from Chromium's DEPS file.
      new_rev = cr_deps_entry.revision
      assert libyuv_deps_entry.url == cr_deps_entry.url, (
        'Libyuv DEPS entry %s has a different URL (%s) than Chromium (%s).' %
        (path, libyuv_deps_entry.url, cr_deps_entry.url))
    else:
      # Use the HEAD of the deps repo.
      stdout, _ = _RunCommand(['git', 'ls-remote', libyuv_deps_entry.url,
                               'HEAD'])
      new_rev = stdout.strip().split('\t')[0]

    # Check if an update is necessary.
    if libyuv_deps_entry.revision != new_rev:
      logging.debug('Roll dependency %s to %s', path, new_rev)
      result.append(ChangedDep(path, libyuv_deps_entry.url,
                               libyuv_deps_entry.revision, new_rev))
  return sorted(result)


def CalculateChangedClang(new_cr_rev):
  def GetClangRev(lines):
    for line in lines:
      match = CLANG_REVISION_RE.match(line)
      if match:
        return match.group(1)
    raise RollError('Could not parse Clang revision from:\n' + '\n'.join('  ' + l for l in lines))

  with open(CLANG_UPDATE_SCRIPT_LOCAL_PATH, 'r') as f:
    current_lines = f.readlines()
  current_rev = GetClangRev(current_lines)

  new_clang_update_py = ReadRemoteCrFile(CLANG_UPDATE_SCRIPT_URL_PATH,
                                             new_cr_rev).splitlines()
  new_rev = GetClangRev(new_clang_update_py)
  return ChangedDep(CLANG_UPDATE_SCRIPT_LOCAL_PATH, None, current_rev, new_rev)


def GenerateCommitMessage(current_cr_rev, new_cr_rev, current_commit_pos,
                          new_commit_pos, changed_deps_list, clang_change):
  current_cr_rev = current_cr_rev[0:10]
  new_cr_rev = new_cr_rev[0:10]
  rev_interval = '%s..%s' % (current_cr_rev, new_cr_rev)
  git_number_interval = '%s:%s' % (current_commit_pos, new_commit_pos)

  commit_msg = ['Roll chromium_revision %s (%s)\n' % (rev_interval,
                                                    git_number_interval)]
  commit_msg.append('Change log: %s' % (CHROMIUM_LOG_TEMPLATE % rev_interval))
  commit_msg.append('Full diff: %s\n' % (CHROMIUM_COMMIT_TEMPLATE %
                                         rev_interval))
  if changed_deps_list:
    commit_msg.append('Changed dependencies:')

    for c in changed_deps_list:
      commit_msg.append('* %s: %s/+log/%s..%s' % (c.path, c.url,
                                                  c.current_rev[0:10],
                                                  c.new_rev[0:10]))
    change_url = CHROMIUM_FILE_TEMPLATE % (rev_interval, 'DEPS')
    commit_msg.append('DEPS diff: %s\n' % change_url)
  else:
    commit_msg.append('No dependencies changed.')

  if clang_change.current_rev != clang_change.new_rev:
    commit_msg.append('Clang version changed %s:%s' %
                      (clang_change.current_rev, clang_change.new_rev))
    change_url = CHROMIUM_FILE_TEMPLATE % (rev_interval,
                                           CLANG_UPDATE_SCRIPT_URL_PATH)
    commit_msg.append('Details: %s\n' % change_url)
  else:
    commit_msg.append('No update to Clang.\n')

  # TBR needs to be non-empty for Gerrit to process it.
  git_author = _RunCommand(['git', 'config', 'user.email'],
                           working_dir=CHECKOUT_SRC_DIR)[0].strip()
  commit_msg.append('TBR=%s' % git_author)

  commit_msg.append('BUG=None')
  return '\n'.join(commit_msg)


def UpdateDepsFile(deps_filename, old_cr_revision, new_cr_revision,
                   changed_deps):
  """Update the DEPS file with the new revision."""

  # Update the chromium_revision variable.
  with open(deps_filename, 'rb') as deps_file:
    deps_content = deps_file.read().decode('utf-8')
  deps_content = deps_content.replace(old_cr_revision, new_cr_revision)
  with open(deps_filename, 'wb') as deps_file:
    deps_file.write(deps_content.encode('utf-8'))

  # Update each individual DEPS entry.
  for dep in changed_deps:
    local_dep_dir = os.path.join(CHECKOUT_ROOT_DIR, dep.path)
    if not os.path.isdir(local_dep_dir):
      raise RollError(
          'Cannot find local directory %s. Make sure the .gclient file\n'
          'contains all platforms in the target_os list, i.e.\n'
          'target_os = ["android", "unix", "mac", "ios", "win"];\n'
          'Then run "gclient sync" again.' % local_dep_dir)
    _RunCommand(
      ['gclient', 'setdep', '--revision', '%s@%s' % (dep.path, dep.new_rev)],
      working_dir=CHECKOUT_SRC_DIR)


def _IsTreeClean():
  stdout, _ = _RunCommand(['git', 'status', '--porcelain'])
  if len(stdout) == 0:
    return True

  logging.error('Dirty/unversioned files:\n%s', stdout)
  return False


def _EnsureUpdatedMasterBranch(dry_run):
  current_branch = _RunCommand(
      ['git', 'rev-parse', '--abbrev-ref', 'HEAD'])[0].splitlines()[0]
  if current_branch != 'main':
    logging.error('Please checkout the main branch and re-run this script.')
    if not dry_run:
      sys.exit(-1)

  logging.info('Updating main branch...')
  _RunCommand(['git', 'pull'])


def _CreateRollBranch(dry_run):
  logging.info('Creating roll branch: %s', ROLL_BRANCH_NAME)
  if not dry_run:
    _RunCommand(['git', 'checkout', '-b', ROLL_BRANCH_NAME])


def _RemovePreviousRollBranch(dry_run):
  active_branch, branches = _GetBranches()
  if active_branch == ROLL_BRANCH_NAME:
    active_branch = 'main'
  if ROLL_BRANCH_NAME in branches:
    logging.info('Removing previous roll branch (%s)', ROLL_BRANCH_NAME)
    if not dry_run:
      _RunCommand(['git', 'checkout', active_branch])
      _RunCommand(['git', 'branch', '-D', ROLL_BRANCH_NAME])


def _LocalCommit(commit_msg, dry_run):
  logging.info('Committing changes locally.')
  if not dry_run:
    _RunCommand(['git', 'add', '--update', '.'])
    _RunCommand(['git', 'commit', '-m', commit_msg])


def ChooseCQMode(skip_cq, cq_over, current_commit_pos, new_commit_pos):
  if skip_cq:
    return 0
  if (new_commit_pos - current_commit_pos) < cq_over:
    return 1
  return 2


def _UploadCL(commit_queue_mode):
  """Upload the committed changes as a changelist to Gerrit.

  commit_queue_mode:
    - 2: Submit to commit queue.
    - 1: Run trybots but do not submit to CQ.
    - 0: Skip CQ, upload only.
  """
  cmd = ['git', 'cl', 'upload', '--force', '--bypass-hooks', '--send-mail']
  if commit_queue_mode >= 2:
    logging.info('Sending the CL to the CQ...')
    cmd.extend(['-o', 'label=Bot-Commit+1'])
    cmd.extend(['-o', 'label=Commit-Queue+2'])
  elif commit_queue_mode >= 1:
    logging.info('Starting CQ dry run...')
    cmd.extend(['-o', 'label=Commit-Queue+1'])
  extra_env = {
      'EDITOR': 'true',
      'SKIP_GCE_AUTH_FOR_GIT': '1',
  }
  stdout, stderr = _RunCommand(cmd, extra_env=extra_env)
  logging.debug('Output from "git cl upload":\nstdout:\n%s\n\nstderr:\n%s',
      stdout, stderr)


def main():
  p = argparse.ArgumentParser()
  p.add_argument('--clean', action='store_true', default=False,
                 help='Removes any previous local roll branch.')
  p.add_argument('-r', '--revision',
                 help=('Chromium Git revision to roll to. Defaults to the '
                       'Chromium HEAD revision if omitted.'))
  p.add_argument('--dry-run', action='store_true', default=False,
                 help=('Calculate changes and modify DEPS, but don\'t create '
                       'any local branch, commit, upload CL or send any '
                       'tryjobs.'))
  p.add_argument('-i', '--ignore-unclean-workdir', action='store_true',
                 default=False,
                 help=('Ignore if the current branch is not main or if there '
                       'are uncommitted changes (default: %(default)s).'))
  grp = p.add_mutually_exclusive_group()
  grp.add_argument('--skip-cq', action='store_true', default=False,
                   help='Skip sending the CL to the CQ (default: %(default)s)')
  grp.add_argument('--cq-over', type=int, default=1,
                   help=('Commit queue dry run if the revision difference '
                         'is below this number (default: %(default)s)'))
  p.add_argument('-v', '--verbose', action='store_true', default=False,
                 help='Be extra verbose in printing of log messages.')
  opts = p.parse_args()

  if opts.verbose:
    logging.basicConfig(level=logging.DEBUG)
  else:
    logging.basicConfig(level=logging.INFO)

  if not opts.ignore_unclean_workdir and not _IsTreeClean():
    logging.error('Please clean your local checkout first.')
    return 1

  if opts.clean:
    _RemovePreviousRollBranch(opts.dry_run)

  if not opts.ignore_unclean_workdir:
    _EnsureUpdatedMasterBranch(opts.dry_run)

  new_cr_rev = opts.revision
  if not new_cr_rev:
    stdout, _ = _RunCommand(['git', 'ls-remote', CHROMIUM_SRC_URL, 'HEAD'])
    head_rev = stdout.strip().split('\t')[0]
    logging.info('No revision specified. Using HEAD: %s', head_rev)
    new_cr_rev = head_rev

  deps_filename = os.path.join(CHECKOUT_SRC_DIR, 'DEPS')
  libyuv_deps = ParseLocalDepsFile(deps_filename)
  current_cr_rev = libyuv_deps['vars']['chromium_revision']

  current_commit_pos = ParseCommitPosition(ReadRemoteCrCommit(current_cr_rev))
  new_commit_pos = ParseCommitPosition(ReadRemoteCrCommit(new_cr_rev))

  new_cr_deps = ParseRemoteCrDepsFile(new_cr_rev)
  changed_deps = CalculateChangedDeps(libyuv_deps, new_cr_deps)
  clang_change = CalculateChangedClang(new_cr_rev)
  commit_msg = GenerateCommitMessage(current_cr_rev, new_cr_rev,
                                     current_commit_pos, new_commit_pos,
                                     changed_deps, clang_change)
  logging.debug('Commit message:\n%s', commit_msg)

  _CreateRollBranch(opts.dry_run)
  UpdateDepsFile(deps_filename, current_cr_rev, new_cr_rev, changed_deps)
  _LocalCommit(commit_msg, opts.dry_run)
  commit_queue_mode = ChooseCQMode(opts.skip_cq, opts.cq_over,
                                   current_commit_pos, new_commit_pos)
  logging.info('Uploading CL...')
  if not opts.dry_run:
    _UploadCL(commit_queue_mode)
  return 0


if __name__ == '__main__':
  sys.exit(main())



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Unit tests for the signal_processing module.
"""

import unittest

import numpy as np
import pydub

from . import exceptions
from . import signal_processing


class TestSignalProcessing(unittest.TestCase):
    """Unit tests for the signal_processing module.
  """

    def testMixSignals(self):
        # Generate a template signal with which white noise can be generated.
        silence = pydub.AudioSegment.silent(duration=1000, frame_rate=48000)

        # Generate two distinct AudioSegment instances with 1 second of white noise.
        signal = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            silence)
        noise = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            silence)

        # Extract samples.
        signal_samples = signal.get_array_of_samples()
        noise_samples = noise.get_array_of_samples()

        # Test target SNR -Inf (noise expected).
        mix_neg_inf = signal_processing.SignalProcessingUtils.MixSignals(
            signal, noise, -np.Inf)
        self.assertTrue(len(noise), len(mix_neg_inf))  # Check duration.
        mix_neg_inf_samples = mix_neg_inf.get_array_of_samples()
        self.assertTrue(  # Check samples.
            all([x == y for x, y in zip(noise_samples, mix_neg_inf_samples)]))

        # Test target SNR 0.0 (different data expected).
        mix_0 = signal_processing.SignalProcessingUtils.MixSignals(
            signal, noise, 0.0)
        self.assertTrue(len(signal), len(mix_0))  # Check duration.
        self.assertTrue(len(noise), len(mix_0))
        mix_0_samples = mix_0.get_array_of_samples()
        self.assertTrue(
            any([x != y for x, y in zip(signal_samples, mix_0_samples)]))
        self.assertTrue(
            any([x != y for x, y in zip(noise_samples, mix_0_samples)]))

        # Test target SNR +Inf (signal expected).
        mix_pos_inf = signal_processing.SignalProcessingUtils.MixSignals(
            signal, noise, np.Inf)
        self.assertTrue(len(signal), len(mix_pos_inf))  # Check duration.
        mix_pos_inf_samples = mix_pos_inf.get_array_of_samples()
        self.assertTrue(  # Check samples.
            all([x == y for x, y in zip(signal_samples, mix_pos_inf_samples)]))

    def testMixSignalsMinInfPower(self):
        silence = pydub.AudioSegment.silent(duration=1000, frame_rate=48000)
        signal = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            silence)

        with self.assertRaises(exceptions.SignalProcessingException):
            _ = signal_processing.SignalProcessingUtils.MixSignals(
                signal, silence, 0.0)

        with self.assertRaises(exceptions.SignalProcessingException):
            _ = signal_processing.SignalProcessingUtils.MixSignals(
                silence, signal, 0.0)

    def testMixSignalNoiseDifferentLengths(self):
        # Test signals.
        shorter = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            pydub.AudioSegment.silent(duration=1000, frame_rate=8000))
        longer = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            pydub.AudioSegment.silent(duration=2000, frame_rate=8000))

        # When the signal is shorter than the noise, the mix length always equals
        # that of the signal regardless of whether padding is applied.
        # No noise padding, length of signal less than that of noise.
        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=shorter,
            noise=longer,
            pad_noise=signal_processing.SignalProcessingUtils.MixPadding.
            NO_PADDING)
        self.assertEqual(len(shorter), len(mix))
        # With noise padding, length of signal less than that of noise.
        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=shorter,
            noise=longer,
            pad_noise=signal_processing.SignalProcessingUtils.MixPadding.
            ZERO_PADDING)
        self.assertEqual(len(shorter), len(mix))

        # When the signal is longer than the noise, the mix length depends on
        # whether padding is applied.
        # No noise padding, length of signal greater than that of noise.
        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=longer,
            noise=shorter,
            pad_noise=signal_processing.SignalProcessingUtils.MixPadding.
            NO_PADDING)
        self.assertEqual(len(shorter), len(mix))
        # With noise padding, length of signal greater than that of noise.
        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=longer,
            noise=shorter,
            pad_noise=signal_processing.SignalProcessingUtils.MixPadding.
            ZERO_PADDING)
        self.assertEqual(len(longer), len(mix))

    def testMixSignalNoisePaddingTypes(self):
        # Test signals.
        shorter = signal_processing.SignalProcessingUtils.GenerateWhiteNoise(
            pydub.AudioSegment.silent(duration=1000, frame_rate=8000))
        longer = signal_processing.SignalProcessingUtils.GeneratePureTone(
            pydub.AudioSegment.silent(duration=2000, frame_rate=8000), 440.0)

        # Zero padding: expect pure tone only in 1-2s.
        mix_zero_pad = signal_processing.SignalProcessingUtils.MixSignals(
            signal=longer,
            noise=shorter,
            target_snr=-6,
            pad_noise=signal_processing.SignalProcessingUtils.MixPadding.
            ZERO_PADDING)

        # Loop: expect pure tone plus noise in 1-2s.
        mix_loop = signal_processing.SignalProcessingUtils.MixSignals(
            signal=longer,
            noise=shorter,
            target_snr=-6,
            pad_noise=signal_processing.SignalProcessingUtils.MixPadding.LOOP)

        def Energy(signal):
            samples = signal_processing.SignalProcessingUtils.AudioSegmentToRawData(
                signal).astype(np.float32)
            return np.sum(samples * samples)

        e_mix_zero_pad = Energy(mix_zero_pad[-1000:])
        e_mix_loop = Energy(mix_loop[-1000:])
        self.assertLess(0, e_mix_zero_pad)
        self.assertLess(e_mix_zero_pad, e_mix_loop)

    def testMixSignalSnr(self):
        # Test signals.
        tone_low = signal_processing.SignalProcessingUtils.GeneratePureTone(
            pydub.AudioSegment.silent(duration=64, frame_rate=8000), 250.0)
        tone_high = signal_processing.SignalProcessingUtils.GeneratePureTone(
            pydub.AudioSegment.silent(duration=64, frame_rate=8000), 3000.0)

        def ToneAmplitudes(mix):
            """Returns the amplitude of the coefficients #16 and #192, which
         correspond to the tones at 250 and 3k Hz respectively."""
            mix_fft = np.absolute(
                signal_processing.SignalProcessingUtils.Fft(mix))
            return mix_fft[16], mix_fft[192]

        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=tone_low, noise=tone_high, target_snr=-6)
        ampl_low, ampl_high = ToneAmplitudes(mix)
        self.assertLess(ampl_low, ampl_high)

        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=tone_high, noise=tone_low, target_snr=-6)
        ampl_low, ampl_high = ToneAmplitudes(mix)
        self.assertLess(ampl_high, ampl_low)

        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=tone_low, noise=tone_high, target_snr=6)
        ampl_low, ampl_high = ToneAmplitudes(mix)
        self.assertLess(ampl_high, ampl_low)

        mix = signal_processing.SignalProcessingUtils.MixSignals(
            signal=tone_high, noise=tone_low, target_snr=6)
        ampl_low, ampl_high = ToneAmplitudes(mix)
        self.assertLess(ampl_low, ampl_high)



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.

from __future__ import division

import logging
import os
import subprocess
import shutil
import sys
import tempfile

try:
    import numpy as np
except ImportError:
    logging.critical('Cannot import the third-party Python package numpy')
    sys.exit(1)

from . import signal_processing


class ExternalVad(object):
    def __init__(self, path_to_binary, name):
        """Args:
       path_to_binary: path to binary that accepts '-i <wav>', '-o
          <float probabilities>'. There must be one float value per
          10ms audio
       name: a name to identify the external VAD. Used for saving
          the output as extvad_output-<name>.
    """
        self._path_to_binary = path_to_binary
        self.name = name
        assert os.path.exists(self._path_to_binary), (self._path_to_binary)
        self._vad_output = None

    def Run(self, wav_file_path):
        _signal = signal_processing.SignalProcessingUtils.LoadWav(
            wav_file_path)
        if _signal.channels != 1:
            raise NotImplementedError('Multiple-channel'
                                      ' annotations not implemented')
        if _signal.frame_rate != 48000:
            raise NotImplementedError('Frame rates '
                                      'other than 48000 not implemented')

        tmp_path = tempfile.mkdtemp()
        try:
            output_file_path = os.path.join(tmp_path, self.name + '_vad.tmp')
            subprocess.call([
                self._path_to_binary, '-i', wav_file_path, '-o',
                output_file_path
            ])
            self._vad_output = np.fromfile(output_file_path, np.float32)
        except Exception as e:
            logging.error('Error while running the ' + self.name + ' VAD (' +
                          e.message + ')')
        finally:
            if os.path.exists(tmp_path):
                shutil.rmtree(tmp_path)

    def GetVadOutput(self):
        assert self._vad_output is not None
        return self._vad_output

    @classmethod
    def ConstructVadDict(cls, vad_paths, vad_names):
        external_vads = {}
        for path, name in zip(vad_paths, vad_names):
            external_vads[name] = ExternalVad(path, name)
        return external_vads



# Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
#
# Use of this source code is governed by a BSD-style license
# that can be found in the LICENSE file in the root of the source
# tree. An additional intellectual property rights grant can be found
# in the file PATENTS.  All contributing project authors may
# be found in the AUTHORS file in the root of the source tree.
"""Echo path simulation module.
"""

import hashlib
import os

from . import signal_processing


class EchoPathSimulator(object):
    """Abstract class for the echo path simulators.

  In general, an echo path simulator is a function of the render signal and
  simulates the propagation of the latter into the microphone (e.g., due to
  mechanical or electrical paths).
  """

    NAME = None
    REGISTERED_CLASSES = {}

    def __init__(self):
        pass

    def Simulate(self, output_path):
        """Creates the echo signal and stores it in an audio file (abstract method).

    Args:
      output_path: Path in which any output can be saved.

    Returns:
      Path to the generated audio track file or None if no echo is present.
    """
        raise NotImplementedError()

    @classmethod
    def RegisterClass(cls, class_to_register):
        """Registers an EchoPathSimulator implementation.

    Decorator to automatically register the classes that extend
    EchoPathSimulator.
    Example usage:

    @EchoPathSimulator.RegisterClass
    class NoEchoPathSimulator(EchoPathSimulator):
      pass
    """
        cls.REGISTERED_CLASSES[class_to_register.NAME] = class_to_register
        return class_to_register


@EchoPathSimulator.RegisterClass
class NoEchoPathSimulator(EchoPathSimulator):
    """Simulates absence of echo."""

    NAME = 'noecho'

    def __init__(self):
        EchoPathSimulator.__init__(self)

    def Simulate(self, output_path):
        return None


@EchoPathSimulator.RegisterClass
class LinearEchoPathSimulator(EchoPathSimulator):
    """Simulates linear echo path.

  This class applies a given impulse response to the render input and then it
  sums the signal to the capture input signal.
  """

    NAME = 'linear'

    def __init__(self, render_input_filepath, impulse_response):
        """
    Args:
      render_input_filepath: Render audio track file.
      impulse_response: list or numpy vector of float values.
    """
        EchoPathSimulator.__init__(self)
        self._render_input_filepath = render_input_filepath
        self._impulse_response = impulse_response

    def Simulate(self, output_path):
        """Simulates linear echo path."""
        # Form the file name with a hash of the impulse response.
        impulse_response_hash = hashlib.sha256(
            str(self._impulse_response).encode('utf-8', 'ignore')).hexdigest()
        echo_filepath = os.path.join(
            output_path, 'linear_echo_{}.wav'.format(impulse_response_hash))

        # If the simulated echo audio track file does not exists, create it.
        if not os.path.exists(echo_filepath):
            render = signal_processing.SignalProcessingUtils.LoadWav(
                self._render_input_filepath)
            echo = signal_processing.SignalProcessingUtils.ApplyImpulseResponse(
                render, self._impulse_response)
            signal_processing.SignalProcessingUtils.SaveWav(
                echo_filepath, echo)

        return echo_filepath


@EchoPathSimulator.RegisterClass
class RecordedEchoPathSimulator(EchoPathSimulator):
    """Uses recorded echo.

  This class uses the clean capture input file name to build the file name of
  the corresponding recording containing echo (a predefined suffix is used).
  Such a file is expected to be already existing.
  """

    NAME = 'recorded'

    _FILE_NAME_SUFFIX = '_echo'

    def __init__(self, render_input_filepath):
        EchoPathSimulator.__init__(self)
        self._render_input_filepath = render_input_filepath

    def Simulate(self, output_path):
        """Uses recorded echo path."""
        path, file_name_ext = os.path.split(self._render_input_filepath)
        file_name, file_ext = os.path.splitext(file_name_ext)
        echo_filepath = os.path.join(
            path, '{}{}{}'.format(file_name, self._FILE_NAME_SUFFIX, file_ext))
        assert os.path.exists(echo_filepath), (
            'cannot find the echo audio track file {}'.format(echo_filepath))
        return echo_filepath
